"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const set_1 = require("./set");
class ring_t {
    constructor(the) {
        this.addition = the.addition;
        this.multiplication = the.multiplication;
        this.elements = the.addition.elements;
    }
    get zero() { return this.addition.id; }
    add(x, y) { return this.addition.add(x, y); }
    neg(x) { return this.addition.neg(x); }
    sub(x, y) { return this.addition.sub(x, y); }
    get one() { return this.multiplication.id; }
    mul(x, y) { return this.multiplication.mul(x, y); }
    eq(x, y) { return this.elements.eq(x, y); }
    zero_p(x) {
        return this.eq(x, this.zero);
    }
    one_p(x) {
        return this.eq(x, this.one);
    }
    left_distr(x, y, z) {
        set_1.eqv(this.elements, this.mul(x, this.add(y, z)), this.add(this.mul(x, y), this.mul(x, z)));
    }
    right_distr(x, y, z) {
        set_1.eqv(this.elements, this.mul(this.add(y, z), x), this.add(this.mul(y, x), this.mul(z, x)));
    }
}
exports.ring_t = ring_t;
class commutative_ring_t extends ring_t {
    commu(x, y) {
        set_1.eqv(this.elements, this.mul(x, y), this.mul(y, x));
    }
    distr(x, y, z) {
        this.left_distr(x, y, z);
        this.right_distr(x, y, z);
    }
}
exports.commutative_ring_t = commutative_ring_t;
class integral_ring_t extends commutative_ring_t {
    nonzero_product(x, y) {
        set_1.not_eqv(this.elements, x, this.zero);
        set_1.not_eqv(this.elements, y, this.zero);
        set_1.not_eqv(this.elements, this.mul(x, y), this.zero);
    }
}
exports.integral_ring_t = integral_ring_t;
class euclidean_ring_t extends integral_ring_t {
    constructor(the) {
        super(the);
        this.degree_lt = the.degree_lt;
        this.divmod = the.divmod;
    }
    degree_lte(x, y) {
        return this.eq(x, y) || this.degree_lt(x, y);
    }
    degree_gt(x, y) {
        return this.degree_lt(y, x);
    }
    degree_gte(x, y) {
        return this.eq(x, y) || this.degree_gt(x, y);
    }
    euclidean_divmod(x, y) {
        set_1.not_eqv(this.elements, y, this.zero);
        let [q, r] = this.divmod(x, y);
        set_1.eqv(this.elements, x, this.add(this.mul(y, q), r));
        assert_1.default(this.eq(r, this.zero) ||
            this.degree_lt(r, y));
    }
    div(x, y) {
        let [q, r] = this.divmod(x, y);
        return q;
    }
    mod(x, y) {
        let [q, r] = this.divmod(x, y);
        return r;
    }
    gcd(x, y) {
        while (!this.eq(y, this.zero)) {
            if (this.degree_lt(x, y)) {
                [x, y] = [y, x];
            }
            else {
                let r = this.mod(x, y);
                [x, y] = [y, r];
            }
        }
        return x;
    }
    gcd_ext(x, y) {
        let old_s = this.one;
        let old_t = this.zero;
        let old_r = x;
        let s = this.zero;
        let t = this.one;
        let r = y;
        while (!this.zero_p(r)) {
            if (this.degree_lt(old_r, r)) {
                [
                    s, t, r,
                    old_s, old_t, old_r,
                ] = [
                    old_s, old_t, old_r,
                    s, t, r,
                ];
            }
            else {
                let q = this.div(old_r, r);
                [old_r, r] = [r, this.sub(old_r, this.mul(q, r))];
                [old_s, s] = [s, this.sub(old_s, this.mul(q, s))];
                [old_t, t] = [t, this.sub(old_t, this.mul(q, t))];
            }
        }
        return [old_r, [old_s, old_t]];
    }
    gcd_ext_p(x, y, res) {
        let [d, [s, t]] = res;
        return (this.eq(d, this.gcd(x, y)) &&
            this.eq(d, this.add(this.mul(s, x), this.mul(t, y))));
    }
}
exports.euclidean_ring_t = euclidean_ring_t;
class field_t extends integral_ring_t {
    constructor(the) {
        super(the);
        this.inv = (x) => {
            set_1.not_eqv(this.elements, x, this.zero);
            return the.inv(x);
        };
    }
    div(x, y) {
        return this.mul(x, this.inv(y));
    }
}
exports.field_t = field_t;
//# sourceMappingURL=ring.js.map