export declare class figure_t {
    array: Array<{
        src: cell_t;
        tar: cell_t;
        cfg: cell_t;
    }>;
    constructor(array?: Array<{
        src: cell_t;
        tar: cell_t;
        cfg: cell_t;
    }>);
    vertex(src: vertex_t, tar: vertex_t): void;
    edge(src: edge_t, tar: edge_t): void;
    edge_rev(src: edge_t, tar: edge_t): void;
    has_src(cell: cell_t): boolean;
    has_tar(cell: cell_t): boolean;
    repr(): any;
    static from_polygon_and_circuit(polygon: polygon_t, cod: cell_complex_t, circuit: circuit_t): figure_t;
    eq(that: figure_t): boolean;
}
export declare class morphism_t {
    dom: cell_complex_t;
    cod: cell_complex_t;
    fig: figure_t;
    uuid: string;
    constructor(the: {
        dom: cell_complex_t;
        cod: cell_complex_t;
        fig?: figure_t;
        uuid?: string;
    });
    eq(that: morphism_t): boolean;
    repr(): any;
}
export declare class cell_t extends morphism_t {
    dom: spherical_t;
    constructor(the: {
        dom: cell_complex_t;
        cod: cell_complex_t;
        fig?: figure_t;
        spherical_evidence?: spherical_evidence_t;
    });
    get dim(): number;
    ser(com: cell_complex_t): number;
}
export declare class cell_complex_t {
    dim: number;
    cell_map: Map<number, Array<cell_t>>;
    name_map: Map<string, cell_t>;
    constructor(the: {
        dim: number;
        cell_map?: Map<number, Array<cell_t>>;
        name_map?: Map<string, cell_t>;
    });
    eq(that: cell_complex_t): boolean;
    skeleton(dim: number): cell_complex_t;
    cell_array(dim: number): Array<cell_t>;
    dim_size(dim: number): number;
    cells(): IterableIterator<cell_t>;
    get vertex_array(): Array<vertex_t>;
    get edge_array(): Array<edge_t>;
    has_name(name: string): boolean;
    has_cell(x: cell_t): boolean;
    attach(name: string, cell: cell_t): void;
    attach_vertex(name: string): void;
    attach_vertexes(names: Array<string>): void;
    attach_edge(name: string, [start, end]: [string, string]): void;
    attach_face(name: string, circuit_exp: Array<edge_exp_t>): void;
    cell(name: string): cell_t;
    vertex(name: string): vertex_t;
    edge(name: string): edge_t;
    face(name: string): face_t;
    empty_p(): boolean;
    repr(): any;
}
export declare class isomorphism_t extends morphism_t {
    constructor(the: {
        dom: cell_complex_t;
        cod: cell_complex_t;
        fig?: figure_t;
    });
    static endpoints(com: cell_complex_t): isomorphism_t | undefined;
    static polygon(com: cell_complex_t): isomorphism_t | undefined;
}
export declare class epimorphism_t extends morphism_t {
    constructor(the: {
        dom: cell_complex_t;
        cod: cell_complex_t;
        fig?: figure_t;
    });
}
export declare class monomorphism_t extends morphism_t {
    constructor(the: {
        dom: cell_complex_t;
        cod: cell_complex_t;
        fig?: figure_t;
    });
}
export declare function isomorphism_p(the: {
    dom: cell_complex_t;
    cod: cell_complex_t;
    fig: figure_t;
}): boolean;
export declare function epimorphism_p(the: {
    dom: cell_complex_t;
    cod: cell_complex_t;
    fig: figure_t;
}): boolean;
export declare function monomorphism_p(the: {
    dom: cell_complex_t;
    cod: cell_complex_t;
    fig: figure_t;
}): boolean;
export declare function spherical_check(evid: spherical_evidence_t, com: cell_complex_t): boolean;
export declare class spherical_t extends cell_complex_t {
    spherical_evidence: spherical_evidence_t;
    constructor(com: cell_complex_t, evid?: spherical_evidence_t);
}
export declare class spherical_evidence_t {
    iso: isomorphism_t;
    constructor(iso: isomorphism_t);
    get dim(): number;
    static from_iso(iso: isomorphism_t | undefined): spherical_evidence_t | undefined;
    static generate(com: cell_complex_t): spherical_evidence_t | undefined;
}
/**
 * A n-dim manifold, is a topological space,
 * each vertex of which has a n-ball as close neighbourhood.

 * For cell-complex, it is sufficient to check that,
 * each vertexes of it has a (n-1)-sphere as [[vertex_figure_t]].
 */
export declare class manifold_evidence_t {
    dim: number;
    constructor(dim: number);
}
export declare class empty_complex_t extends cell_complex_t {
    constructor();
}
export declare class empty_morphism_t extends morphism_t {
    constructor();
}
export declare class vertex_t extends cell_t {
    constructor();
}
export declare class endpoints_t extends cell_complex_t {
    constructor();
}
export declare class edge_t extends cell_t {
    start: vertex_t;
    end: vertex_t;
    endpoints: endpoints_t;
    constructor(com: cell_complex_t, start: vertex_t, end: vertex_t);
    get rev(): edge_rev_t;
}
export declare class polygon_t extends cell_complex_t {
    size: number;
    constructor(size: number);
}
/**
 * not recursive type like sexp.
 */
export declare type edge_exp_t = string | Array<string>;
export declare class edge_rev_t {
    rev: edge_t;
    constructor(rev: edge_t);
}
export declare function edge_exp_eval(com: cell_complex_t, exp: edge_exp_t): edge_t | edge_rev_t;
declare type circuit_t = Array<edge_t | edge_rev_t>;
export declare class face_t extends cell_t {
    circuit: circuit_t;
    polygon: polygon_t;
    constructor(com: cell_complex_t, circuit: circuit_t);
}
export {};
