"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
class game_t {
    valid_choice_p(p, c, s) {
        let choices = this.choices(p, s);
        return choices.some((x) => _.isEqual(x, c));
    }
}
exports.game_t = game_t;
class play_t {
    constructor(game, init_state, init_player, players, moments = []) {
        this.game = game;
        this.init_state = init_state;
        this.init_player = init_player;
        this.players = players;
        this.moments = moments;
    }
    beginning_p() {
        return this.moments.length === 0;
    }
    last_moment() {
        if (this.beginning_p()) {
            throw new Error("play_t.last_moment");
        }
        else {
            let moment = this.moments.slice(-1).pop();
            return moment;
        }
    }
    last_player() { return this.last_moment().player; }
    last_choice() { return this.last_moment().choice; }
    last_state() {
        if (this.beginning_p()) {
            return this.init_state;
        }
        else {
            return this.last_moment().state;
        }
    }
    tow_player_alternating() {
        if (this.players.length !== 2) {
            throw new Error("play_t.tow_player_alternating fail");
        }
        let p0 = this.players[0];
        let p1 = this.players[1];
        if (this.beginning_p()) {
            return this.init_player;
        }
        else {
            switch (this.last_player()) {
                case p0: return p1;
                case p1: return p0;
                default: throw new Error("play_t.tow_player_alternating fail");
            }
        }
    }
    winner() {
        let s = this.last_state();
        for (let p of this.players) {
            if (this.game.win_p(p, s)) {
                return p;
            }
        }
        return null;
    }
    state_log(s) {
        console.log(s);
    }
    move(p, c) {
        if (this.next_player() === p) {
            let s = this.last_state();
            if (this.game.valid_choice_p(p, c, s)) {
                let next_state = this.game.choose(p, c, s);
                this.moments.push({
                    player: p,
                    choice: c,
                    state: next_state,
                });
            }
            else {
                console.log(`[warning]`, `invalid move: ${c} for player: ${p}`);
            }
        }
        else {
            console.log(`[warning]`, `next player should be ${this.next_player()}`);
        }
        this.state_log(this.last_state());
        let w = this.winner();
        if (w !== null)
            console.log(`[info]`, `winner is ${w}`);
    }
}
exports.play_t = play_t;
//# sourceMappingURL=combinatorial-game.js.map