"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Map with `string` as key.
 */
class dic_t {
    constructor(key_to_str = JSON.stringify) {
        this.val_map = new Map();
        this.key_map = new Map();
        this.key_to_str = key_to_str;
    }
    set(k, v) {
        let s = this.key_to_str(k);
        this.val_map.set(s, v);
        this.key_map.set(s, k);
    }
    has(k) {
        let s = this.key_to_str(k);
        return this.val_map.has(s);
    }
    get(k) {
        let s = this.key_to_str(k);
        let v = this.val_map.get(s);
        if (v === undefined) {
            throw new Error("key no in dic");
        }
        return v;
    }
    update_at(k, f) {
        let v = this.get(k);
        this.set(k, f(v));
        return this;
    }
    get size() {
        return this.val_map.size;
    }
    empty_p() {
        return this.size === 0;
    }
    to_array() {
        let array = new Array();
        for (let [s, v] of this.val_map) {
            let k = this.key_map.get(s);
            array.push([k, v]);
        }
        return array;
    }
    merge_array(array) {
        for (let [k, v] of array) {
            this.set(k, v);
        }
        return this;
    }
    *[Symbol.iterator]() {
        for (let [s, v] of this.val_map) {
            let k = this.key_map.get(s);
            yield [k, v];
        }
    }
    *entries() {
        for (let [s, v] of this.val_map) {
            let k = this.key_map.get(s);
            yield [k, v];
        }
    }
    *keys() {
        for (let k of this.key_map.values()) {
            yield k;
        }
    }
    *values() {
        for (let v of this.val_map.values()) {
            yield v;
        }
    }
    /**
     * Assuming `this.key_lt (that)`.
     */
    *zip(that) {
        for (let k of this.keys()) {
            let x = this.get(k);
            let y = that.get(k);
            yield [k, [x, y]];
        }
    }
    key_array() {
        return new Array(...this.key_map.values());
    }
    value_array() {
        return new Array(...this.val_map.values());
    }
    copy() {
        let dic = new dic_t(this.key_to_str);
        dic.val_map = new Map(this.val_map);
        dic.key_map = new Map(this.key_map);
        return dic;
    }
    key_eq(that) {
        if (this.size !== that.size) {
            return false;
        }
        for (let k of this.keys()) {
            if (!that.has(k)) {
                return false;
            }
        }
        return true;
    }
    key_lt(that) {
        if (!(this.size < that.size)) {
            return false;
        }
        for (let k of this.keys()) {
            if (!that.has(k)) {
                return false;
            }
        }
        return true;
    }
    key_lteq(that) {
        if (!(this.size <= that.size)) {
            return false;
        }
        for (let k of this.keys()) {
            if (!that.has(k)) {
                return false;
            }
        }
        return true;
    }
    key_gt(that) {
        return that.key_lt(this);
    }
    key_gteq(that) {
        return that.key_lteq(this);
    }
}
exports.dic_t = dic_t;
//# sourceMappingURL=dic.js.map