import { permutation_t } from "./permutation";
export declare type Array1d = Array<number>;
export declare type Array2d = Array<Array<number>>;
export declare type Array3d = Array<Array<Array<number>>>;
export declare function argmax(lo: number, hi: number, f: (i: number) => number): number;
export declare function argmax_guard(lo: number, hi: number, f: (i: number) => number, p: (i: number) => boolean): number;
export declare function argmin(lo: number, hi: number, f: (i: number) => number): number;
export declare function argmin_guard(lo: number, hi: number, f: (i: number) => number, p: (i: number) => boolean): number;
export declare function argfirst(lo: number, hi: number, p: (i: number) => boolean): number | null;
export declare function argall(lo: number, hi: number, p: (i: number) => boolean): boolean;
export declare function argsome(lo: number, hi: number, p: (i: number) => boolean): boolean;
export interface config_t {
    /** for almost degenerated matrix */
    epsilon: number;
}
export declare let config: config_t;
export declare function epsilon_p(x: number): boolean;
export declare function non_epsilon_p(x: number): boolean;
export declare class matrix_t {
    readonly buffer: Float64Array;
    readonly shape: [number, number];
    readonly strides: [number, number];
    readonly offset: number;
    readonly size: number;
    constructor(buffer: Float64Array, shape: [number, number], strides: [number, number], offset?: number);
    static shape_to_size([x, y]: [number, number]): number;
    static shape_to_strides([x, y]: [number, number]): [number, number];
    static from_buffer(buffer: Float64Array, shape: [number, number]): matrix_t;
    static fromArray2d(array: Array2d): matrix_t;
    static from_row(row: vector_t): matrix_t;
    static from_col(col: vector_t): matrix_t;
    get_linear_index(x: number, y: number): number;
    get(x: number, y: number): number;
    set(x: number, y: number, v: number): matrix_t;
    update_at(i: number, j: number, f: (v: number) => number): matrix_t;
    static indexes_of_shape([m, n]: [number, number]): Generator<[number, number], void, unknown>;
    indexes(): Generator<[number, number], void, unknown>;
    entries(): Generator<[number, number, number], void, unknown>;
    values(): Generator<number, void, unknown>;
    update_op1(op: (x: number) => number): matrix_t;
    same_shape_p(that: matrix_t): boolean;
    update_op2(that: matrix_t, op: (x: number, y: number) => number): matrix_t;
    slice(x: [number, number] | null, y: [number, number] | null): matrix_t;
    copy(): matrix_t;
    update_copy(that: matrix_t): matrix_t;
    row(i: number): vector_t;
    set_row(i: number, src: vector_t): matrix_t;
    row_entries(): Generator<[number, vector_t], void, unknown>;
    rows(): Generator<vector_t, void, unknown>;
    for_each_row_index(f: (row: vector_t, i: number) => any): matrix_t;
    for_each_row(f: (row: vector_t) => any): matrix_t;
    col(j: number): vector_t;
    set_col(j: number, src: vector_t): matrix_t;
    col_entries(): Generator<[number, vector_t], void, unknown>;
    cols(): Generator<vector_t, void, unknown>;
    for_each_col_index(f: (col: vector_t, i: number) => any): matrix_t;
    for_each_col(f: (col: vector_t) => any): matrix_t;
    reduce_with(init: number, f: (acc: number, cur: number) => number): number;
    every(p: (v: number) => boolean): boolean;
    some(p: (v: number) => boolean): boolean;
    zip(that: matrix_t): Generator<number[], void, unknown>;
    eq(that: matrix_t): boolean;
    toArray2d(): Array2d;
    print(): void;
    update_add(that: matrix_t): matrix_t;
    add(that: matrix_t): matrix_t;
    update_sub(that: matrix_t): matrix_t;
    sub(that: matrix_t): matrix_t;
    map(f: (v: number) => number): matrix_t;
    update_scale(a: number): matrix_t;
    scale(a: number): matrix_t;
    mul(that: matrix_t): matrix_t;
    act(v: vector_t): vector_t;
    transpose(): matrix_t;
    symmetric_p(): boolean;
    square_p(): boolean;
    update_swap_rows(i: number, j: number): matrix_t;
    update_swap_cols(i: number, j: number): matrix_t;
    append_cols(that: matrix_t): matrix_t;
    append_rows(that: matrix_t): matrix_t;
    /**
     * The main diag.
     */
    diag(): vector_t;
    static constant(n: number, x: number, y: number): matrix_t;
    static zeros(x: number, y: number): matrix_t;
    static ones(x: number, y: number): matrix_t;
    static id(n: number): matrix_t;
    static row_trans(permutation: permutation_t): matrix_t;
    static col_trans(permutation: permutation_t): matrix_t;
    tuck_row(i: number, j: number): matrix_t;
    tuck_col(i: number, j: number): matrix_t;
    update_tuck_row(i: number, j: number): matrix_t;
    update_tuck_col(i: number, j: number): matrix_t;
    row_echelon_form(): matrix_t;
    /**
     * with all pivots equal to 1.
     */
    unit_row_echelon_form(): matrix_t;
    /**
     * unit row echelon form + back substitution
  
     * The reduced row echelon form of a matrix is unique
     * i.e. does not depend on the algorithm used to compute it.
     */
    reduced_row_echelon_form(): matrix_t;
    upper_p(): boolean;
    lower_p(): boolean;
    upper(): matrix_t;
    strict_upper(): matrix_t;
    lower(): matrix_t;
    strict_lower(): matrix_t;
    /**
     * P * A = L * U,
     * `permu.mul (this) .eq (lower.mul (upper))`,
     * (singular matrixes allowed)
     */
    lower_upper_decomposition(): {
        lower: matrix_t;
        upper: matrix_t;
        permu: matrix_t;
        inver: number;
    };
    rank(): number;
    singular_p(): boolean;
    non_singular_p(): boolean;
    invertible_p(): boolean;
    inv_maybe(): matrix_t | null;
    inv(): matrix_t;
    det(): number;
    epsilon_p(): boolean;
    /**
     * The same as `reduced_row_echelon_form`.
     */
    row_canonical_form(): matrix_t;
    /**
     * `row_trans.mul (this) .eq (canonical)`
     */
    row_canonical_decomposition(): {
        row_trans: matrix_t;
        canonical: matrix_t;
    };
    zeros_rows_at_bottom_p(r: number): boolean;
    has_row_pivots_p(): boolean;
    row_pivot_indexes(): Generator<[number, number], void, unknown>;
    /**
     * (1) zeros rows at bottom
     * (2) non zeros has leading pivots from left to right
     * (3) in pivot columns all other elements are zeros
     */
    row_canonical_form_p(): boolean;
    /**
     * Bases of column space, represented by columns of matrix.
     */
    image(): matrix_t;
    /**
     * Bases of null space, represented by columns of matrix.
     */
    kernel(): matrix_t;
    solve(b: vector_t): null | vector_t;
    row_hermite_normal_form(): matrix_t;
    integral_p(): boolean;
    /**
     * (1) zeros rows at bottom
     * (2) non zeros has leading pivots from left to right
     * (3) in pivot columns all other elements are less than pivot
     */
    row_hermite_form_p(): boolean;
    det_abs_one_p(): boolean;
    /**
     * `row_trans.mul (this) .eq (hermite)`
     */
    row_hermite_decomposition(): {
        row_trans: matrix_t;
        hermite: matrix_t;
    };
    int_image(): matrix_t;
    int_kernel(): matrix_t;
    int_solve(b: vector_t): null | vector_t;
    int_solve_matrix(matrix: matrix_t): null | matrix_t;
    smith_update(): matrix_t;
    invariant_factor_update(): matrix_t;
    diag_abs_update(): matrix_t;
    diagonal_p(): boolean;
    smith_normal_form_p(): boolean;
    smith_normal_form(): matrix_t;
    smith_update_ext(the: {
        row_trans: matrix_t;
        col_trans: matrix_t;
    }): void;
    invariant_factor_update_ext(the: {
        row_trans: matrix_t;
        col_trans: matrix_t;
    }): void;
    diag_abs_update_ext(the: {
        row_trans: matrix_t;
    }): void;
    /**
     * `row_trans.mul (this) .mul (col_trans) .eq (smith)`
     */
    smith_decomposition(): {
        row_trans: matrix_t;
        col_trans: matrix_t;
        smith: matrix_t;
    };
}
export declare function matrix(array: Array2d): matrix_t;
/**
 * Although `Array` in js is written as a row,
 * `vector_t` should be viewed as column vector.
 * (respecting the tradition of mathematics)
 *
 * `A x` is a combination of the columns of `A`.
 */
export declare class vector_t {
    readonly buffer: Float64Array;
    readonly shape: [number];
    readonly strides: [number];
    readonly offset: number;
    readonly size: number;
    constructor(buffer: Float64Array, shape: [number], strides: [number], offset?: number);
    static from_buffer(buffer: Float64Array): vector_t;
    static fromArray(array: Array1d): vector_t;
    get_linear_index(x: number): number;
    get(i: number): number;
    set(i: number, v: number): vector_t;
    toArray(): Array<number>;
    print(): void;
    indexes(): Generator<number, void, unknown>;
    update_at(i: number, f: (v: number) => number): vector_t;
    update(f: (v: number) => number): vector_t;
    values(): Generator<number, void, unknown>;
    entries(): Generator<[number, number], void, unknown>;
    copy(): vector_t;
    zip(that: vector_t): Generator<number[], void, unknown>;
    eq(that: vector_t): boolean;
    dot(that: vector_t): number;
    map(f: (v: number) => number): vector_t;
    scale(a: number): vector_t;
    update_scale(a: number): vector_t;
    static constant(n: number, size: number): vector_t;
    static zeros(size: number): vector_t;
    static ones(size: number): vector_t;
    trans(matrix: matrix_t): vector_t;
    append(that: vector_t): vector_t;
    some(p: (v: number) => boolean): boolean;
    every(p: (v: number) => boolean): boolean;
    update_add(that: vector_t): vector_t;
    add(that: vector_t): vector_t;
    update_sub(that: vector_t): vector_t;
    sub(that: vector_t): vector_t;
    reduce_with(init: number, f: (acc: number, cur: number) => number): number;
    reduce(f: (acc: number, cur: number) => number): number;
    argfirst(p: (x: number) => boolean): number | null;
    first(p: (x: number) => boolean): number | null;
    argmax(f: (x: number) => number): number;
    invariant_factors_p(): boolean;
    zero_p(): boolean;
    one_p(): boolean;
    epsilon_p(): boolean;
    integral_p(): boolean;
}
export declare function vector(array: Array1d): vector_t;
