"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const assert_1 = __importDefault(require("assert"));
const ut = __importStar(require("./util"));
const num = __importStar(require("./number"));
const permutation_1 = require("./permutation");
function argmax(lo, hi, f) {
    let arg = lo;
    let cur = f(lo);
    for (let i = lo; i < hi; i++) {
        let next = f(i);
        if (cur < next) {
            arg = i;
            cur = next;
        }
    }
    return arg;
}
exports.argmax = argmax;
function argmax_guard(lo, hi, f, p) {
    let arg = argfirst(lo, hi, p);
    if (arg === null) {
        throw new Error("no such arg");
    }
    let cur = f(lo);
    for (let i = arg + 1; i < hi; i++) {
        if (p(i)) {
            let next = f(i);
            if (cur < next) {
                arg = i;
                cur = next;
            }
        }
    }
    return arg;
}
exports.argmax_guard = argmax_guard;
function argmin(lo, hi, f) {
    let arg = lo;
    let cur = f(lo);
    for (let i = lo; i < hi; i++) {
        let next = f(i);
        if (cur > next) {
            arg = i;
            cur = next;
        }
    }
    return arg;
}
exports.argmin = argmin;
function argmin_guard(lo, hi, f, p) {
    let arg = argfirst(lo, hi, p);
    if (arg === null) {
        throw new Error("no such arg");
    }
    let cur = f(lo);
    for (let i = arg + 1; i < hi; i++) {
        if (p(i)) {
            let next = f(i);
            if (cur > next) {
                arg = i;
                cur = next;
            }
        }
    }
    return arg;
}
exports.argmin_guard = argmin_guard;
function argfirst(lo, hi, p) {
    for (let i = lo; i < hi; i++) {
        if (p(i)) {
            return i;
        }
    }
    return null;
}
exports.argfirst = argfirst;
function argall(lo, hi, p) {
    for (let i = lo; i < hi; i++) {
        if (!p(i)) {
            return false;
        }
    }
    return true;
}
exports.argall = argall;
function argsome(lo, hi, p) {
    for (let i = lo; i < hi; i++) {
        if (p(i)) {
            return true;
        }
    }
    return false;
}
exports.argsome = argsome;
exports.config = {
    epsilon: 0.000000001
};
function epsilon_p(x) {
    return Math.abs(x) < exports.config.epsilon;
}
exports.epsilon_p = epsilon_p;
function non_epsilon_p(x) {
    return Math.abs(x) >= exports.config.epsilon;
}
exports.non_epsilon_p = non_epsilon_p;
class matrix_t {
    constructor(buffer, shape, strides, offset = 0) {
        this.buffer = buffer;
        this.shape = shape;
        this.strides = strides;
        this.offset = offset;
        this.size = matrix_t.shape_to_size(shape);
    }
    static shape_to_size([x, y]) {
        return x * y;
    }
    static shape_to_strides([x, y]) {
        return [y, 1];
    }
    static from_buffer(buffer, shape) {
        let strides = matrix_t.shape_to_strides(shape);
        return new matrix_t(buffer, shape, strides);
    }
    static fromArray2d(array) {
        let y_length = array.length;
        let x_length = array[0].length;
        for (let a of array) {
            if (a.length !== x_length) {
                throw new Error("inner array length mismatch");
            }
        }
        let buffer = Float64Array.from(array.flat());
        return matrix_t.from_buffer(buffer, [y_length, x_length]);
    }
    static from_row(row) {
        let [s] = row.strides;
        return new matrix_t(row.buffer, [1, row.size], [0, s], row.offset);
    }
    static from_col(col) {
        let [t] = col.strides;
        return new matrix_t(col.buffer, [col.size, 1], [t, 0], col.offset);
    }
    get_linear_index(x, y) {
        return (this.offset +
            x * this.strides[0] +
            y * this.strides[1]);
    }
    get(x, y) {
        return this.buffer[this.get_linear_index(x, y)];
    }
    set(x, y, v) {
        this.buffer[this.get_linear_index(x, y)] = v;
        return this;
    }
    update_at(i, j, f) {
        this.set(i, j, f(this.get(i, j)));
        return this;
    }
    static *indexes_of_shape([m, n]) {
        for (let i = 0; i < m; i++) {
            for (let j = 0; j < n; j++) {
                yield [i, j];
            }
        }
    }
    *indexes() {
        for (let [i, j] of matrix_t.indexes_of_shape(this.shape)) {
            yield [i, j];
        }
    }
    *entries() {
        for (let [i, j] of this.indexes()) {
            let v = this.get(i, j);
            yield [i, j, v];
        }
    }
    *values() {
        for (let [i, j] of this.indexes()) {
            let v = this.get(i, j);
            yield v;
        }
    }
    update_op1(op) {
        let [m, n] = this.shape;
        for (let i = 0; i < m; i++) {
            for (let j = 0; j < n; j++) {
                let v = op(this.get(i, j));
                this.set(i, j, v);
            }
        }
        return this;
    }
    same_shape_p(that) {
        return ((this.shape[0] === that.shape[0]) &&
            (this.shape[1] === that.shape[1]));
    }
    update_op2(that, op) {
        assert_1.default(this.same_shape_p(that));
        let [m, n] = this.shape;
        for (let i = 0; i < m; i++) {
            for (let j = 0; j < n; j++) {
                let v = op(this.get(i, j), that.get(i, j));
                this.set(i, j, v);
            }
        }
        return this;
    }
    slice(x, y) {
        let [m, n] = this.shape;
        let offset = this.offset;
        if (x !== null) {
            let [start, end] = x;
            m = end - start;
            offset += start * this.strides[0];
        }
        if (y !== null) {
            let [start, end] = y;
            n = end - start;
            offset += start * this.strides[1];
        }
        return new matrix_t(this.buffer, [m, n], this.strides, offset);
    }
    copy() {
        let buffer = new Float64Array(this.size);
        let matrix = matrix_t.from_buffer(buffer, this.shape);
        for (let [i, j, x] of this.entries()) {
            matrix.set(i, j, x);
        }
        return matrix;
    }
    update_copy(that) {
        let [m, n] = this.shape;
        for (let i of ut.range(0, m)) {
            for (let j of ut.range(0, n)) {
                this.set(i, j, that.get(i, j));
            }
        }
        return this;
    }
    row(i) {
        let [m, n] = this.shape;
        let [s, t] = this.strides;
        let offset = this.offset + i * s;
        return new vector_t(this.buffer, [n], [t], offset);
    }
    set_row(i, src) {
        let [m, n] = this.shape;
        for (let j of ut.range(0, n)) {
            this.set(i, j, src.get(j));
        }
        return this;
    }
    *row_entries() {
        let [m, _n] = this.shape;
        for (let i = 0; i < m; i++) {
            yield [i, this.row(i)];
        }
    }
    *rows() {
        let [m, _n] = this.shape;
        for (let i = 0; i < m; i++) {
            yield this.row(i);
        }
    }
    for_each_row_index(f) {
        for (let [i, row] of this.row_entries()) {
            f(row, i);
        }
        return this;
    }
    for_each_row(f) {
        for (let row of this.rows()) {
            f(row);
        }
        return this;
    }
    col(j) {
        let [m, n] = this.shape;
        let [s, t] = this.strides;
        let offset = this.offset + j * t;
        return new vector_t(this.buffer, [m], [s], offset);
    }
    set_col(j, src) {
        let [m, n] = this.shape;
        for (let i of ut.range(0, m)) {
            this.set(i, j, src.get(i));
        }
        return this;
    }
    *col_entries() {
        let [_m, n] = this.shape;
        for (let i = 0; i < n; i++) {
            yield [i, this.col(i)];
        }
    }
    *cols() {
        let [_m, n] = this.shape;
        for (let i = 0; i < n; i++) {
            yield this.col(i);
        }
    }
    for_each_col_index(f) {
        for (let [i, col] of this.col_entries()) {
            f(col, i);
        }
        return this;
    }
    for_each_col(f) {
        for (let col of this.cols()) {
            f(col);
        }
        return this;
    }
    reduce_with(init, f) {
        let acc = init;
        for (let v of this.values()) {
            acc = f(acc, v);
        }
        return acc;
    }
    every(p) {
        for (let v of this.values()) {
            if (!p(v)) {
                return false;
            }
        }
        return true;
    }
    some(p) {
        for (let v of this.values()) {
            if (p(v)) {
                return true;
            }
        }
        return false;
    }
    *zip(that) {
        let this_iter = this.values();
        let that_iter = that.values();
        while (true) {
            let this_next = this_iter.next();
            let that_next = that_iter.next();
            if (this_next.done || that_next.done) {
                return;
            }
            else {
                yield [this_next.value, that_next.value];
            }
        }
    }
    eq(that) {
        if (this.size !== that.size) {
            return false;
        }
        if (!_.isEqual(this.shape, that.shape)) {
            return false;
        }
        for (let [x, y] of this.zip(that)) {
            if (x !== y) {
                return false;
            }
        }
        return true;
    }
    toArray2d() {
        let array = [];
        let [x, _] = this.shape;
        for (let i = 0; i < x; i++) {
            array.push(this.row(i).toArray());
        }
        return array;
    }
    print() {
        console.log("matrix:");
        console.table(this.toArray2d());
    }
    update_add(that) {
        return this.update_op2(that, (x, y) => x + y);
    }
    add(that) {
        return this.copy().update_add(that);
    }
    update_sub(that) {
        return this.update_op2(that, (x, y) => x - y);
    }
    sub(that) {
        return this.copy().update_sub(that);
    }
    map(f) {
        let matrix = this.copy();
        for (let [i, j, v] of this.entries()) {
            matrix.update_at(i, j, f);
        }
        return matrix;
    }
    update_scale(a) {
        return this.update_op1(x => x * a);
    }
    scale(a) {
        return this.map(n => n * a);
    }
    mul(that) {
        let [m, n] = this.shape;
        let [p, q] = that.shape;
        assert_1.default(n === p);
        let shape = [m, q];
        let size = matrix_t.shape_to_size(shape);
        let buffer = new Float64Array(size);
        let matrix = matrix_t.from_buffer(buffer, shape);
        for (let i of ut.range(0, m)) {
            for (let j of ut.range(0, q)) {
                let v = this.row(i).dot(that.col(j));
                matrix.set(i, j, v);
            }
        }
        return matrix;
    }
    act(v) {
        return v.trans(this);
    }
    transpose() {
        let [m, n] = this.shape;
        let [s, t] = this.strides;
        return new matrix_t(this.buffer, [n, m], [t, s], this.offset);
    }
    symmetric_p() {
        return this.eq(this.transpose());
    }
    square_p() {
        let [x, y] = this.shape;
        return x === y;
    }
    update_swap_rows(i, j) {
        let [m, n] = this.shape;
        for (let k of ut.range(0, n)) {
            let x = this.get(i, k);
            let y = this.get(j, k);
            this.set(i, k, y);
            this.set(j, k, x);
        }
        return this;
    }
    update_swap_cols(i, j) {
        let [m, n] = this.shape;
        for (let k of ut.range(0, m)) {
            let x = this.get(k, i);
            let y = this.get(k, j);
            this.set(k, i, y);
            this.set(k, j, x);
        }
        return this;
    }
    append_cols(that) {
        let [m, n] = this.shape;
        let [p, q] = that.shape;
        assert_1.default(m === p);
        let shape = [m, n + q];
        let size = matrix_t.shape_to_size(shape);
        let buffer = new Float64Array(size);
        let matrix = matrix_t.from_buffer(buffer, shape);
        for (let i of ut.range(0, m)) {
            let row = this.row(i).append(that.row(i));
            matrix.set_row(i, row);
        }
        return matrix;
    }
    append_rows(that) {
        let [m, n] = this.shape;
        let [p, q] = that.shape;
        assert_1.default(n === q);
        let shape = [m + p, n];
        let size = matrix_t.shape_to_size(shape);
        let buffer = new Float64Array(size);
        let matrix = matrix_t.from_buffer(buffer, shape);
        for (let i of ut.range(0, n)) {
            let col = this.col(i).append(that.col(i));
            matrix.set_col(i, col);
        }
        return matrix;
    }
    /**
     * The main diag.
     */
    diag() {
        let [m, n] = this.shape;
        n = Math.min(m, n);
        let vector = vector_t.zeros(n);
        for (let i of ut.range(0, n)) {
            vector.set(i, this.get(i, i));
        }
        return vector;
    }
    static constant(n, x, y) {
        let shape = [x, y];
        let size = matrix_t.shape_to_size(shape);
        let buffer = new Float64Array(size);
        buffer.fill(n);
        return matrix_t.from_buffer(buffer, shape);
    }
    static zeros(x, y) {
        return matrix_t.constant(0, x, y);
    }
    static ones(x, y) {
        return matrix_t.constant(1, x, y);
    }
    static id(n) {
        let matrix = matrix_t.zeros(n, n);
        for (let i of ut.range(0, n)) {
            matrix.set(i, i, 1);
        }
        return matrix;
    }
    static row_trans(permutation) {
        let n = permutation.size;
        let matrix = matrix_t.zeros(n, n);
        for (let [i, v] of permutation.pairs()) {
            matrix.set(i, v, 1);
        }
        return matrix;
    }
    static col_trans(permutation) {
        let n = permutation.size;
        let matrix = matrix_t.zeros(n, n);
        for (let [i, v] of permutation.pairs()) {
            matrix.set(v, i, 1);
        }
        return matrix;
    }
    tuck_row(i, j) {
        let [m, n] = this.shape;
        let row_trans = matrix_t.row_trans(permutation_1.permutation_t.id(m).tuck(i, j));
        return row_trans.mul(this);
    }
    tuck_col(i, j) {
        let [m, n] = this.shape;
        let col_trans = matrix_t.col_trans(permutation_1.permutation_t.id(n).tuck(i, j));
        return this.mul(col_trans);
    }
    update_tuck_row(i, j) {
        this.update_copy(this.tuck_row(i, j));
        return this;
    }
    update_tuck_col(i, j) {
        this.update_copy(this.tuck_col(i, j));
        return this;
    }
    row_echelon_form() {
        let matrix = this.copy();
        let [m, n] = this.shape;
        let h = 0; // init pivot row
        let k = 0; // init pivot column
        while (h < m && k < n) {
            // find the next pivot
            let piv = argmax(h, m, (i) => Math.abs(matrix.get(i, k)));
            if (epsilon_p(matrix.get(piv, k))) {
                // no pivot in this column, pass to next column
                k += 1;
            }
            else {
                if (h !== piv) {
                    matrix.update_swap_rows(h, piv);
                }
                // for all rows below pivot
                for (let i = h + 1; i < m; i++) {
                    let f = matrix.get(i, k) / matrix.get(h, k);
                    matrix.set(i, k, 0);
                    // for all remaining elements in current row
                    for (let j = k + 1; j < n; j++) {
                        let v = matrix.get(i, j) - matrix.get(h, j) * f;
                        matrix.set(i, j, v);
                    }
                }
                h += 1;
                k += 1;
            }
        }
        return matrix;
    }
    /**
     * with all pivots equal to 1.
     */
    unit_row_echelon_form() {
        let matrix = this.row_echelon_form();
        matrix.for_each_row_index((row, i) => {
            let pivot = row.first(x => !epsilon_p(x));
            if (pivot !== null) {
                row.update_scale(1 / pivot);
            }
        });
        return matrix;
    }
    /**
     * unit row echelon form + back substitution
  
     * The reduced row echelon form of a matrix is unique
     * i.e. does not depend on the algorithm used to compute it.
     */
    reduced_row_echelon_form() {
        let matrix = this.unit_row_echelon_form();
        for (let [i, row] of matrix.row_entries()) {
            for (let [j, sub] of matrix.row_entries()) {
                if (j > i) {
                    let arg = sub.argfirst(x => x === 1);
                    if (arg !== null) {
                        let x = matrix.get(i, arg);
                        if (!epsilon_p(x)) {
                            row.update_add(sub.scale(-x));
                        }
                    }
                }
            }
        }
        return matrix;
    }
    upper_p() {
        for (let [i, j, v] of this.entries()) {
            if (i > j) {
                if (!epsilon_p(v)) {
                    return false;
                }
            }
        }
        return true;
    }
    lower_p() {
        for (let [i, j, v] of this.entries()) {
            if (i < j) {
                if (!epsilon_p(v)) {
                    return false;
                }
            }
        }
        return true;
    }
    upper() {
        let matrix = this.copy();
        for (let [i, j] of this.indexes()) {
            if (i > j) {
                matrix.set(i, j, 0);
            }
        }
        return matrix;
    }
    strict_upper() {
        let matrix = this.copy();
        for (let [i, j] of this.indexes()) {
            if (i >= j) {
                matrix.set(i, j, 0);
            }
        }
        return matrix;
    }
    lower() {
        let matrix = this.copy();
        for (let [i, j] of this.indexes()) {
            if (i < j) {
                matrix.set(i, j, 0);
            }
        }
        return matrix;
    }
    strict_lower() {
        let matrix = this.copy();
        for (let [i, j] of this.indexes()) {
            if (i <= j) {
                matrix.set(i, j, 0);
            }
        }
        return matrix;
    }
    /**
     * P * A = L * U,
     * `permu.mul (this) .eq (lower.mul (upper))`,
     * (singular matrixes allowed)
     */
    lower_upper_decomposition() {
        let matrix = this.copy();
        let [m, n] = this.shape;
        assert_1.default(m === n);
        let record = matrix_t.zeros(m, n);
        let permu = matrix_t.id(n);
        let h = 0; // init pivot row
        let k = 0; // init pivot column
        let inver = 0;
        while (h < m && k < n) {
            // find the next pivot
            let piv = argmax(h, m, (i) => Math.abs(matrix.get(i, k)));
            if (epsilon_p(matrix.get(piv, k))) {
                // no pivot in this column, pass to next column
                k += 1;
            }
            else {
                if (h !== piv) {
                    matrix.update_swap_rows(h, piv);
                    record.update_swap_rows(h, piv);
                    permu.update_swap_rows(h, piv);
                    inver += 1;
                }
                // for all rows below pivot
                for (let i = h + 1; i < m; i++) {
                    let f = matrix.get(i, k) / matrix.get(h, k);
                    matrix.set(i, k, 0);
                    record.update_at(i, k, v => v + f);
                    // for all remaining elements in current row
                    for (let j = k + 1; j < n; j++) {
                        let v = matrix.get(i, j) - matrix.get(h, j) * f;
                        matrix.set(i, j, v);
                    }
                }
                h += 1;
                k += 1;
            }
        }
        return {
            lower: record.update_add(matrix_t.id(n)),
            upper: matrix,
            permu,
            inver,
        };
    }
    rank() {
        let echelon = this.row_echelon_form();
        let rank = 0;
        for (let row of echelon.rows()) {
            if (row.some(v => !epsilon_p(v))) {
                rank += 1;
            }
        }
        return rank;
    }
    singular_p() {
        assert_1.default(this.square_p());
        let [_m, n] = this.shape;
        for (let row of this.rows()) {
            if (row.every(x => epsilon_p(x))) {
                return true;
            }
        }
        for (let col of this.cols()) {
            if (col.every(x => epsilon_p(x))) {
                return true;
            }
        }
        return this.rank() < n;
    }
    non_singular_p() {
        return !this.singular_p();
    }
    invertible_p() {
        return this.non_singular_p();
    }
    inv_maybe() {
        assert_1.default(this.square_p());
        let [_m, n] = this.shape;
        let augmented = this.append_cols(matrix_t.id(n));
        let echelon = augmented.reduced_row_echelon_form();
        let upper = echelon.slice(null, [0, n]);
        let inv = echelon.slice(null, [n, n + n]);
        if (upper.singular_p()) {
            return null;
        }
        else {
            return inv;
        }
    }
    inv() {
        let inv = this.inv_maybe();
        if (inv === null) {
            throw new Error("not invertible");
        }
        else {
            return inv;
        }
    }
    det() {
        assert_1.default(this.square_p());
        let { lower, upper, permu, inver } = this.lower_upper_decomposition();
        let sign;
        if (inver % 2 === 0) {
            sign = +1;
        }
        else {
            sign = -1;
        }
        return sign * upper.diag().reduce((acc, cur) => acc * cur);
    }
    epsilon_p() {
        return this.every(epsilon_p);
    }
    /**
     * The same as `reduced_row_echelon_form`.
     */
    row_canonical_form() {
        let matrix = this.copy();
        let [m, n] = this.shape;
        let i = 0;
        let j = 0;
        while (i < m && j < n) {
            let k = argmax(i, m, (k) => Math.abs(matrix.get(k, j)));
            if (epsilon_p(matrix.get(k, j))) {
                j += 1;
            }
            else {
                if (k !== i) {
                    matrix.update_swap_rows(i, k);
                }
                matrix.row(i).update_scale(1 / matrix.get(i, j));
                for (let k of ut.ranges([[0, i], [i + 1, m]])) {
                    let v = matrix.get(k, j);
                    if (v !== 0) {
                        matrix.row(k).update_sub(matrix.row(i).scale(v));
                    }
                }
                i += 1;
                j += 1;
            }
        }
        return matrix;
    }
    /**
     * `row_trans.mul (this) .eq (canonical)`
     */
    row_canonical_decomposition() {
        let [m, n] = this.shape;
        let augmented = this.append_cols(matrix_t.id(m));
        let echelon = augmented.row_canonical_form();
        return {
            row_trans: echelon.slice(null, [n, n + m]),
            canonical: echelon.slice(null, [0, n]),
        };
    }
    zeros_rows_at_bottom_p(r) {
        let [m, n] = this.shape;
        for (let i of ut.range(r, m)) {
            let row = this.row(i);
            if (!row.every(epsilon_p)) {
                return false;
            }
        }
        return true;
    }
    has_row_pivots_p() {
        let r = this.rank();
        if (!this.zeros_rows_at_bottom_p(r)) {
            return false;
        }
        let j = 0;
        for (let i of ut.range(0, r)) {
            let arg = this.row(i).argfirst(non_epsilon_p);
            if (arg === null) {
                return false;
            }
            else if (arg >= j) {
                j = arg;
            }
            else {
                return false;
            }
        }
        return true;
    }
    *row_pivot_indexes() {
        assert_1.default(this.has_row_pivots_p());
        let r = this.rank();
        for (let i of ut.range(0, r)) {
            let arg = this.row(i).argfirst(non_epsilon_p);
            if (arg === null) {
                assert_1.default(false);
            }
            else {
                yield [i, arg];
            }
        }
    }
    /**
     * (1) zeros rows at bottom
     * (2) non zeros has leading pivots from left to right
     * (3) in pivot columns all other elements are zeros
     */
    row_canonical_form_p() {
        if (!this.has_row_pivots_p()) {
            return false;
        }
        for (let [i, j] of this.row_pivot_indexes()) {
            for (let k of ut.range(0, i)) {
                if (!epsilon_p(this.get(k, j))) {
                    return false;
                }
            }
        }
        return true;
    }
    /**
     * Bases of column space, represented by columns of matrix.
     */
    image() {
        return this.transpose()
            .row_canonical_form()
            .transpose()
            .slice(null, [0, this.rank()]);
    }
    /**
     * Bases of null space, represented by columns of matrix.
     */
    kernel() {
        let [m, n] = this.shape;
        let r = this.rank();
        let { row_trans } = this.transpose()
            .row_canonical_decomposition();
        return row_trans.transpose().slice(null, [r, n]);
    }
    solve(b) {
        let [m, n] = this.shape;
        let r = this.rank();
        let augmented = this.append_cols(matrix_t.from_col(b))
            .row_canonical_form();
        for (let i of ut.range(r, m)) {
            if (non_epsilon_p(augmented.get(i, n))) {
                return null;
            }
        }
        let vector = vector_t.zeros(n);
        for (let [i, j] of augmented.row_pivot_indexes()) {
            vector.set(j, augmented.get(i, n));
        }
        return vector;
    }
    row_hermite_normal_form() {
        let matrix = this.copy();
        let [m, n] = this.shape;
        let i = 0;
        let j = 0;
        while (i < m && j < n) {
            if (argall(i, m, (k) => matrix.get(k, j) === 0)) {
                j += 1;
            }
            else {
                while (!((argall(i + 1, m, (k) => matrix.get(k, j) === 0) &&
                    argall(0, i, (k) => 0 <= matrix.get(k, j) &&
                        matrix.get(k, j) < matrix.get(i, j))))) {
                    let k = argmin_guard(i, m, (k) => Math.abs(matrix.get(k, j)), (k) => matrix.get(k, j) !== 0);
                    if (k !== i) {
                        matrix.update_swap_rows(i, k);
                    }
                    if (matrix.get(i, j) < 0) {
                        matrix.row(i).update_scale(-1);
                    }
                    for (let k of ut.ranges([[0, i], [i + 1, m]])) {
                        let q = num.div(matrix.get(k, j), matrix.get(i, j));
                        if (q !== 0) {
                            matrix.row(k)
                                .update_sub(matrix.row(i).scale(q));
                        }
                    }
                }
                i += 1;
                j += 1;
            }
        }
        return matrix;
    }
    integral_p() {
        return this.every(Number.isInteger);
    }
    /**
     * (1) zeros rows at bottom
     * (2) non zeros has leading pivots from left to right
     * (3) in pivot columns all other elements are less than pivot
     */
    row_hermite_form_p() {
        if (!this.integral_p()) {
            return false;
        }
        if (!this.has_row_pivots_p()) {
            return false;
        }
        for (let [i, j] of this.row_pivot_indexes()) {
            for (let k of ut.range(0, i)) {
                if (!(this.get(k, j) < this.get(i, j))) {
                    return false;
                }
            }
        }
        return true;
    }
    det_abs_one_p() {
        if (!this.integral_p()) {
            return false;
        }
        if (!this.invertible_p()) {
            return false;
        }
        if (!epsilon_p(Math.abs(this.det()) - 1)) {
            return false;
        }
        return true;
    }
    /**
     * `row_trans.mul (this) .eq (hermite)`
     */
    row_hermite_decomposition() {
        let [m, n] = this.shape;
        let augmented = this.append_cols(matrix_t.id(m));
        let echelon = augmented.row_hermite_normal_form();
        return {
            row_trans: echelon.slice(null, [n, n + m]),
            hermite: echelon.slice(null, [0, n]),
        };
    }
    int_image() {
        return this.transpose()
            .row_hermite_normal_form()
            .transpose()
            .slice(null, [0, this.rank()]);
    }
    int_kernel() {
        let [m, n] = this.shape;
        let r = this.rank();
        let { col_trans } = this.smith_decomposition();
        return col_trans.slice(null, [r, n]);
    }
    int_solve(b) {
        let [m, n] = this.shape;
        assert_1.default(b.size === m);
        let r = this.rank();
        let { row_trans, col_trans, smith, } = this.smith_decomposition();
        let c = b.trans(row_trans);
        let vector = vector_t.zeros(n);
        for (let i of ut.range(0, c.size)) {
            let x = c.get(i);
            let y = i < r ? smith.get(i, i) : 0;
            if (y === 0) {
                if (x !== 0) {
                    return null;
                }
            }
            else {
                if (x % y === 0) {
                    vector.set(i, num.div(x, y));
                }
                else {
                    return null;
                }
            }
        }
        return vector.trans(col_trans);
    }
    int_solve_matrix(matrix) {
        assert_1.default(this.integral_p);
        assert_1.default(matrix.integral_p);
        let [m, n] = this.shape;
        let [p, q] = matrix.shape;
        assert_1.default(p === m);
        let solution = matrix_t.zeros(n, q);
        for (let [i, col] of matrix.col_entries()) {
            let x = this.int_solve(col);
            if (x === null) {
                return null;
            }
            else {
                solution.set_col(i, x);
            }
        }
        return solution;
    }
    smith_update() {
        let matrix = this;
        let [m, n] = this.shape;
        let i = 0;
        let j = 0;
        while (i < m && j < n) {
            if ((argall(i, m, (k) => matrix.get(k, j) === 0) &&
                argall(j, n, (k) => matrix.get(i, k) === 0))) {
                i += 1;
                j += 1;
            }
            else {
                while (!((argall(i + 1, m, (k) => matrix.get(k, j) === 0) &&
                    argall(j + 1, n, (k) => matrix.get(i, k) === 0)))) {
                    // { // debug
                    //   console.log ("main loop")
                    //   matrix.print ()
                    // }
                    // It is amazing that this loop converges.
                    // It is like kneading dough.
                    while (!argall(i + 1, m, (k) => matrix.get(k, j) === 0)) {
                        // { // debug
                        //   console.log ("row elimination loop")
                        // }
                        let k = argmin_guard(i, m, (k) => Math.abs(matrix.get(k, j)), (k) => matrix.get(k, j) !== 0);
                        if (k !== i) {
                            matrix.update_swap_rows(i, k);
                        }
                        if (matrix.get(i, j) < 0) {
                            matrix.row(i).update_scale(-1);
                        }
                        for (let k of ut.range(i + 1, m)) {
                            let q = num.div(matrix.get(k, j), matrix.get(i, j));
                            if (q !== 0) {
                                matrix.row(k)
                                    .update_sub(matrix.row(i).scale(q));
                            }
                        }
                    }
                    while (!argall(j + 1, n, (k) => matrix.get(i, k) === 0)) {
                        // { // debug
                        //   console.log ("col elimination loop")
                        // }
                        let k = argmin_guard(j, n, (k) => Math.abs(matrix.get(i, k)), (k) => matrix.get(i, k) !== 0);
                        if (k !== j) {
                            matrix.update_swap_cols(j, k);
                        }
                        if (matrix.get(i, j) < 0) {
                            matrix.col(j).update_scale(-1);
                        }
                        for (let k of ut.range(j + 1, n)) {
                            let q = num.div(matrix.get(i, k), matrix.get(i, j));
                            if (q !== 0) {
                                matrix.col(k)
                                    .update_sub(matrix.col(j).scale(q));
                            }
                        }
                    }
                }
                i += 1;
                j += 1;
            }
        }
        return matrix;
    }
    invariant_factor_update() {
        let matrix = this;
        let [m, n] = this.shape;
        m = Math.min(m, n);
        let i = 0;
        while (i < m) {
            if (matrix.get(i, i) === 0) {
                for (let k of ut.range(i, m)) {
                    matrix.set(k, k, matrix.get(k + 1, k + 1));
                }
                matrix.set(m - 1, m - 1, 0);
                m -= 1;
            }
            else {
                let x = matrix.get(i, i);
                for (let k of ut.range(i + 1, m)) {
                    let y = matrix.get(k, k);
                    if (y % x !== 0) {
                        matrix.set(k, i, matrix.get(k, k));
                        matrix.smith_update();
                    }
                }
                i += 1;
            }
        }
        return matrix;
    }
    diag_abs_update() {
        let matrix = this;
        let [m, n] = this.shape;
        m = Math.min(m, n);
        for (let i of ut.range(0, m)) {
            matrix.update_at(i, i, x => Math.abs(x));
        }
        return matrix;
    }
    diagonal_p() {
        let [m, n] = this.shape;
        for (let i of ut.range(0, m)) {
            for (let j of ut.range(0, n)) {
                if (i !== j) {
                    if (non_epsilon_p(this.get(i, j))) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    smith_normal_form_p() {
        if (!this.diagonal_p()) {
            return false;
        }
        if (!this.every(x => x >= 0)) {
            return false;
        }
        if (!this.diag().invariant_factors_p()) {
            return false;
        }
        return true;
    }
    smith_normal_form() {
        return this.copy()
            .smith_update()
            .invariant_factor_update()
            .diag_abs_update();
    }
    smith_update_ext(the) {
        let matrix = this;
        let [m, n] = this.shape;
        let row_trans = the.row_trans;
        let col_trans = the.col_trans;
        let i = 0;
        let j = 0;
        while (i < m && j < n) {
            if ((argall(i, m, (k) => matrix.get(k, j) === 0) &&
                argall(j, n, (k) => matrix.get(i, k) === 0))) {
                i += 1;
                j += 1;
            }
            else {
                while (!((argall(i + 1, m, (k) => matrix.get(k, j) === 0) &&
                    argall(j + 1, n, (k) => matrix.get(i, k) === 0)))) {
                    // row_trans
                    while (!argall(i + 1, m, (k) => matrix.get(k, j) === 0)) {
                        let k = argmin_guard(i, m, (k) => Math.abs(matrix.get(k, j)), (k) => matrix.get(k, j) !== 0);
                        if (k !== i) {
                            matrix.update_swap_rows(i, k);
                            row_trans.update_swap_rows(i, k);
                        }
                        if (matrix.get(i, j) < 0) {
                            matrix.row(i).update_scale(-1);
                            row_trans.row(i).update_scale(-1);
                        }
                        for (let k of ut.range(i + 1, m)) {
                            let q = num.div(matrix.get(k, j), matrix.get(i, j));
                            if (q !== 0) {
                                matrix.row(k)
                                    .update_sub(matrix.row(i).scale(q));
                                row_trans.row(k)
                                    .update_sub(row_trans.row(i).scale(q));
                            }
                        }
                    }
                    // col_trans
                    while (!argall(j + 1, n, (k) => matrix.get(i, k) === 0)) {
                        let k = argmin_guard(j, n, (k) => Math.abs(matrix.get(i, k)), (k) => matrix.get(i, k) !== 0);
                        if (k !== j) {
                            matrix.update_swap_cols(j, k);
                            col_trans.update_swap_cols(j, k);
                        }
                        if (matrix.get(i, j) < 0) {
                            matrix.col(j).update_scale(-1);
                            col_trans.col(j).update_scale(-1);
                        }
                        for (let k of ut.range(j + 1, n)) {
                            let q = num.div(matrix.get(i, k), matrix.get(i, j));
                            if (q !== 0) {
                                matrix.col(k)
                                    .update_sub(matrix.col(j).scale(q));
                                col_trans.col(k)
                                    .update_sub(col_trans.col(j).scale(q));
                            }
                        }
                    }
                }
                i += 1;
                j += 1;
            }
        }
    }
    invariant_factor_update_ext(the) {
        let [m, n] = this.shape;
        m = Math.min(m, n);
        let i = 0;
        while (i < m) {
            if (this.get(i, i) === 0) {
                this.update_tuck_row(i, m - 1);
                this.update_tuck_col(i, m - 1);
                the.row_trans.update_tuck_row(i, m - 1);
                the.col_trans.update_tuck_col(i, m - 1);
                m -= 1;
            }
            else {
                let x = this.get(i, i);
                for (let k of ut.range(i + 1, m)) {
                    let y = this.get(k, k);
                    if (y % x !== 0) {
                        this.col(i)
                            .update_add(this.col(k));
                        the.col_trans.col(i)
                            .update_add(the.col_trans.col(k));
                        this.smith_update_ext(the);
                    }
                }
                i += 1;
            }
        }
    }
    diag_abs_update_ext(the) {
        let [m, n] = this.shape;
        m = Math.min(m, n);
        for (let i of ut.range(0, m)) {
            if (this.get(i, i) < 0) {
                this.set(i, i, -this.get(i, i));
                the.row_trans.row(i).update_scale(-1);
            }
        }
    }
    /**
     * `row_trans.mul (this) .mul (col_trans) .eq (smith)`
     */
    smith_decomposition() {
        let [m, n] = this.shape;
        let smith = this.copy();
        let the = {
            row_trans: matrix_t.id(m),
            col_trans: matrix_t.id(n),
        };
        smith.smith_update_ext(the);
        assert_1.default(the.row_trans.invertible_p());
        assert_1.default(the.col_trans.invertible_p());
        smith.invariant_factor_update_ext(the);
        assert_1.default(the.row_trans.invertible_p());
        assert_1.default(the.col_trans.invertible_p());
        smith.diag_abs_update_ext(the);
        assert_1.default(the.row_trans.invertible_p());
        assert_1.default(the.col_trans.invertible_p());
        return { ...the, smith };
    }
}
exports.matrix_t = matrix_t;
function matrix(array) {
    return matrix_t.fromArray2d(array);
}
exports.matrix = matrix;
/**
 * Although `Array` in js is written as a row,
 * `vector_t` should be viewed as column vector.
 * (respecting the tradition of mathematics)
 *
 * `A x` is a combination of the columns of `A`.
 */
class vector_t {
    constructor(buffer, shape, strides, offset = 0) {
        this.buffer = buffer;
        this.shape = shape;
        this.strides = strides;
        this.offset = offset;
        this.size = shape[0];
    }
    static from_buffer(buffer) {
        return new vector_t(buffer, [buffer.length], [1]);
    }
    static fromArray(array) {
        return vector_t.from_buffer(new Float64Array(array));
    }
    get_linear_index(x) {
        return (this.offset +
            x * this.strides[0]);
    }
    get(i) {
        return this.buffer[this.get_linear_index(i)];
    }
    set(i, v) {
        this.buffer[this.get_linear_index(i)] = v;
        return this;
    }
    toArray() {
        let array = [];
        for (let i of ut.range(0, this.size)) {
            array.push(this.get(i));
        }
        return array;
    }
    print() {
        console.log("vector:");
        console.table(this.toArray());
    }
    *indexes() {
        for (let i of ut.range(0, this.size)) {
            yield i;
        }
    }
    update_at(i, f) {
        return this.set(i, f(this.get(i)));
    }
    update(f) {
        for (let i of this.indexes()) {
            this.set(i, f(this.get(i)));
        }
        return this;
    }
    *values() {
        for (let i of ut.range(0, this.size)) {
            let v = this.buffer[this.get_linear_index(i)];
            yield v;
        }
    }
    *entries() {
        for (let i of ut.range(0, this.size)) {
            let v = this.buffer[this.get_linear_index(i)];
            yield [i, v];
        }
    }
    copy() {
        let buffer = new Float64Array(this.size);
        let vector = vector_t.from_buffer(buffer);
        for (let [i, x] of this.entries()) {
            vector.set(i, x);
        }
        return vector;
    }
    *zip(that) {
        let this_iter = this.values();
        let that_iter = that.values();
        while (true) {
            let this_next = this_iter.next();
            let that_next = that_iter.next();
            if (this_next.done || that_next.done) {
                return;
            }
            else {
                yield [this_next.value, that_next.value];
            }
        }
    }
    eq(that) {
        if (this.size !== that.size) {
            return false;
        }
        for (let [x, y] of this.zip(that)) {
            if (x !== y) {
                return false;
            }
        }
        return true;
    }
    dot(that) {
        assert_1.default(this.size === that.size);
        let product = 0;
        for (let [i, y] of that.entries()) {
            product += this.get(i) * y;
        }
        return product;
    }
    map(f) {
        let vector = this.copy();
        for (let [i, v] of this.entries()) {
            vector.update_at(i, f);
        }
        return vector;
    }
    scale(a) {
        return this.map(n => n * a);
    }
    update_scale(a) {
        return this.update(n => n * a);
    }
    static constant(n, size) {
        let buffer = new Float64Array(size).fill(n);
        return vector_t.from_buffer(buffer);
    }
    static zeros(size) {
        return vector_t.constant(0, size);
    }
    static ones(size) {
        return vector_t.constant(1, size);
    }
    trans(matrix) {
        let [m, n] = matrix.shape;
        assert_1.default(n === this.size);
        let vector = vector_t.zeros(m);
        for (let i of ut.range(0, m)) {
            vector.set(i, this.dot(matrix.row(i)));
        }
        return vector;
    }
    append(that) {
        let buffer = new Float64Array(this.size + that.size);
        let vector = vector_t.from_buffer(buffer);
        for (let [i, x] of this.entries()) {
            vector.set(i, x);
        }
        for (let [i, x] of that.entries()) {
            vector.set(i + this.size, x);
        }
        return vector;
    }
    some(p) {
        for (let v of this.values()) {
            if (p(v)) {
                return true;
            }
        }
        return false;
    }
    every(p) {
        for (let v of this.values()) {
            if (!p(v)) {
                return false;
            }
        }
        return true;
    }
    update_add(that) {
        assert_1.default(this.size === that.size);
        for (let [i, x] of that.entries()) {
            this.update_at(i, v => v + x);
        }
        return this;
    }
    add(that) {
        assert_1.default(this.size === that.size);
        let vector = this.copy();
        for (let [i, x] of that.entries()) {
            vector.update_at(i, v => v + x);
        }
        return vector;
    }
    update_sub(that) {
        assert_1.default(this.size === that.size);
        for (let [i, x] of that.entries()) {
            this.update_at(i, v => v - x);
        }
        return this;
    }
    sub(that) {
        assert_1.default(this.size === that.size);
        let vector = this.copy();
        for (let [i, x] of that.entries()) {
            vector.update_at(i, v => v - x);
        }
        return vector;
    }
    reduce_with(init, f) {
        let acc = init;
        for (let v of this.values()) {
            acc = f(acc, v);
        }
        return acc;
    }
    reduce(f) {
        assert_1.default(this.size > 0);
        if (this.size === 1) {
            return this.get(0);
        }
        else {
            let acc = this.get(0);
            for (let i of ut.range(1, this.size)) {
                acc = f(acc, this.get(i));
            }
            return acc;
        }
    }
    argfirst(p) {
        let lo = 0;
        let hi = this.size;
        return argfirst(lo, hi, i => p(this.get(i)));
    }
    first(p) {
        let arg = this.argfirst(p);
        if (arg === null) {
            return null;
        }
        else {
            return this.get(arg);
        }
    }
    argmax(f) {
        let lo = 0;
        let hi = this.size;
        return argmax(lo, hi, i => f(this.get(i)));
    }
    invariant_factors_p() {
        for (let i of ut.range(0, this.size)) {
            let x = this.get(i);
            if (x !== 0) {
                for (let k of ut.range(i + 1, this.size)) {
                    let y = this.get(k);
                    if (y % x !== 0) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    zero_p() {
        return this.every(x => x === 0);
    }
    one_p() {
        return this.every(x => x === 1);
    }
    epsilon_p() {
        return this.every(epsilon_p);
    }
    integral_p() {
        return this.every(Number.isInteger);
    }
}
exports.vector_t = vector_t;
function vector(array) {
    return vector_t.fromArray(array);
}
exports.vector = vector;
//# sourceMappingURL=euclid--naive.js.map