import { permutation_t } from "./permutation";
import { set_t } from "./abstract/set";
import { euclidean_ring_t } from "./abstract/ring";
export declare type Array1d<R> = Array<R>;
export declare type Array2d<R> = Array<Array<R>>;
export declare type Array3d<R> = Array<Array<Array<R>>>;
export declare function argfirst(lo: number, hi: number, p: (i: number) => boolean): number | null;
export declare function argall(lo: number, hi: number, p: (i: number) => boolean): boolean;
export declare function argsome(lo: number, hi: number, p: (i: number) => boolean): boolean;
export declare function argcmp(lo: number, hi: number, c: (i: number, j: number) => boolean): number;
export declare function argcmp_guard(lo: number, hi: number, c: (i: number, j: number) => boolean, p: (i: number) => boolean): number;
export declare function ring<R>(the: {
    elements: set_t<R>;
    zero: R;
    add: (x: R, y: R) => R;
    neg: (x: R) => R;
    one: R;
    mul: (x: R, y: R) => R;
    degree_lt: (x: R, y: R) => boolean;
    divmod: (x: R, y: R) => [R, R];
}): euclidean_ring_t<R>;
export declare class matrix_t<R> {
    readonly ring: euclidean_ring_t<R>;
    readonly buffer: Array<R>;
    readonly shape: [number, number];
    readonly strides: [number, number];
    readonly offset: number;
    readonly size: number;
    constructor(the: {
        ring: euclidean_ring_t<R>;
        buffer: Array<R>;
        shape: [number, number];
        strides: [number, number];
        offset?: number;
    });
    static shape_to_size([x, y]: [number, number]): number;
    static shape_to_strides([x, y]: [number, number]): [number, number];
    static from_ring_buffer<R>(ring: euclidean_ring_t<R>, buffer: Array<R>, shape: [number, number]): matrix_t<R>;
    static from_ring_Array2d<R>(ring: euclidean_ring_t<R>, array: Array2d<R>): matrix_t<R>;
    static from_ring_row<R>(ring: euclidean_ring_t<R>, row: vector_t<R>): matrix_t<R>;
    static from_ring_col<R>(ring: euclidean_ring_t<R>, col: vector_t<R>): matrix_t<R>;
    get_linear_index(x: number, y: number): number;
    get(x: number, y: number): R;
    set(x: number, y: number, v: R): matrix_t<R>;
    update_at(i: number, j: number, f: (v: R) => R): matrix_t<R>;
    static indexes_of_shape([m, n]: [number, number]): Generator<[number, number], void, unknown>;
    indexes(): Generator<[number, number], void, unknown>;
    entries(): Generator<[number, number, R], void, unknown>;
    values(): Generator<R, void, unknown>;
    update_op1(op: (x: R) => R): matrix_t<R>;
    same_shape_p(that: matrix_t<R>): boolean;
    update_op2(that: matrix_t<R>, op: (x: R, y: R) => R): matrix_t<R>;
    slice(x: [number, number] | null, y: [number, number] | null): matrix_t<R>;
    copy(): matrix_t<R>;
    update_copy(that: matrix_t<R>): matrix_t<R>;
    row(i: number): vector_t<R>;
    set_row(i: number, src: vector_t<R>): matrix_t<R>;
    row_entries(): Generator<[number, vector_t<R>], void, unknown>;
    rows(): Generator<vector_t<R>, void, unknown>;
    for_each_row_index(f: (row: vector_t<R>, i: number) => any): matrix_t<R>;
    for_each_row(f: (row: vector_t<R>) => any): matrix_t<R>;
    col(j: number): vector_t<R>;
    set_col(j: number, src: vector_t<R>): matrix_t<R>;
    col_entries(): Generator<[number, vector_t<R>], void, unknown>;
    cols(): Generator<vector_t<R>, void, unknown>;
    for_each_col_index(f: (col: vector_t<R>, i: number) => any): matrix_t<R>;
    for_each_col(f: (col: vector_t<R>) => any): matrix_t<R>;
    reduce_with(init: R, f: (acc: R, cur: R) => R): R;
    every(p: (v: R) => boolean): boolean;
    some(p: (v: R) => boolean): boolean;
    zip(that: matrix_t<R>): Generator<R[], void, unknown>;
    eq(that: matrix_t<R>): boolean;
    toArray2d(): Array2d<R>;
    print(): void;
    update_add(that: matrix_t<R>): matrix_t<R>;
    add(that: matrix_t<R>): matrix_t<R>;
    update_sub(that: matrix_t<R>): matrix_t<R>;
    sub(that: matrix_t<R>): matrix_t<R>;
    map(f: (v: R) => R): matrix_t<R>;
    update_scale(a: R): matrix_t<R>;
    scale(a: R): matrix_t<R>;
    mul(that: matrix_t<R>): matrix_t<R>;
    act(v: vector_t<R>): vector_t<R>;
    transpose(): matrix_t<R>;
    symmetric_p(): boolean;
    square_p(): boolean;
    zero_p(): boolean;
    update_swap_rows(i: number, j: number): matrix_t<R>;
    update_swap_cols(i: number, j: number): matrix_t<R>;
    append_cols(that: matrix_t<R>): matrix_t<R>;
    append_rows(that: matrix_t<R>): matrix_t<R>;
    /**
     * The main diag.
     */
    diag(): vector_t<R>;
    static ring_constant<R>(ring: euclidean_ring_t<R>, n: R, x: number, y: number): matrix_t<R>;
    static ring_zeros<R>(ring: euclidean_ring_t<R>, x: number, y: number): matrix_t<R>;
    static ring_ones<R>(ring: euclidean_ring_t<R>, x: number, y: number): matrix_t<R>;
    static ring_id<R>(ring: euclidean_ring_t<R>, n: number): matrix_t<R>;
    static ring_row_trans<R>(ring: euclidean_ring_t<R>, permutation: permutation_t): matrix_t<R>;
    static ring_col_trans<R>(ring: euclidean_ring_t<R>, permutation: permutation_t): matrix_t<R>;
    tuck_row(i: number, j: number): matrix_t<R>;
    tuck_col(i: number, j: number): matrix_t<R>;
    update_tuck_row(i: number, j: number): matrix_t<R>;
    update_tuck_col(i: number, j: number): matrix_t<R>;
    /**
     * I preserve the terms
     * `hermite_normal_form` and `smith_normal_form`
     * for matrix_t of integers.
     */
    /**
     * Generic `hermite_normal_form`
     */
    row_canonical_form(): matrix_t<R>;
    /**
     * `row_trans.mul (this) .eq (row_canonical)`
     */
    row_canonical_decomposition(): {
        row_trans: matrix_t<R>;
        row_canonical: matrix_t<R>;
    };
    /**
     * Generic `smith_normal_form`
     */
    diag_canonical_update(): matrix_t<R>;
    invariant_factor_update(): matrix_t<R>;
    diag_canonical_form(): matrix_t<R>;
    diagonal_p(): boolean;
    diag_canonical_form_p(): boolean;
    diag_canonical_update_ext(the: {
        row_trans: matrix_t<R>;
        col_trans: matrix_t<R>;
    }): void;
    invariant_factor_update_ext(the: {
        row_trans: matrix_t<R>;
        col_trans: matrix_t<R>;
    }): void;
    /**
     * `row_trans.mul (this) .mul (col_trans) .eq (diag_canonical)`
     */
    diag_canonical_decomposition(): {
        row_trans: matrix_t<R>;
        col_trans: matrix_t<R>;
        diag_canonical: matrix_t<R>;
    };
    rank(): number;
    image(): matrix_t<R>;
    kernel(): matrix_t<R>;
    solve(b: vector_t<R>): null | vector_t<R>;
    solve_matrix(matrix: matrix_t<R>): null | matrix_t<R>;
}
export declare class vector_t<R> {
    readonly ring: euclidean_ring_t<R>;
    readonly buffer: Array<R>;
    readonly shape: [number];
    readonly strides: [number];
    readonly offset: number;
    readonly size: number;
    constructor(the: {
        ring: euclidean_ring_t<R>;
        buffer: Array<R>;
        shape: [number];
        strides: [number];
        offset?: number;
    });
    static from_ring_buffer<R>(ring: euclidean_ring_t<R>, buffer: Array<R>): vector_t<R>;
    static from_ring_Array<R>(ring: euclidean_ring_t<R>, array: Array1d<R>): vector_t<R>;
    get_linear_index(x: number): number;
    get(i: number): R;
    set(i: number, v: R): vector_t<R>;
    toArray(): Array<R>;
    print(): void;
    indexes(): Generator<number, void, unknown>;
    update_at(i: number, f: (v: R) => R): vector_t<R>;
    update(f: (v: R) => R): vector_t<R>;
    values(): Generator<R, void, unknown>;
    entries(): Generator<[number, R], void, unknown>;
    copy(): vector_t<R>;
    zip(that: vector_t<R>): Generator<R[], void, unknown>;
    eq(that: vector_t<R>): boolean;
    dot(that: vector_t<R>): R;
    map(f: (v: R) => R): vector_t<R>;
    scale(a: R): vector_t<R>;
    update_scale(a: R): vector_t<R>;
    static ring_constant<R>(ring: euclidean_ring_t<R>, n: R, size: number): vector_t<R>;
    static ring_zeros<R>(ring: euclidean_ring_t<R>, size: number): vector_t<R>;
    static ring_ones<R>(ring: euclidean_ring_t<R>, size: number): vector_t<R>;
    trans(matrix: matrix_t<R>): vector_t<R>;
    append(that: vector_t<R>): vector_t<R>;
    some(p: (v: R) => boolean): boolean;
    every(p: (v: R) => boolean): boolean;
    update_add(that: vector_t<R>): vector_t<R>;
    add(that: vector_t<R>): vector_t<R>;
    update_sub(that: vector_t<R>): vector_t<R>;
    sub(that: vector_t<R>): vector_t<R>;
    reduce_with(init: R, f: (acc: R, cur: R) => R): R;
    reduce(f: (acc: R, cur: R) => R): R;
    argfirst(p: (x: R) => boolean): number | null;
    first(p: (x: R) => boolean): R | null;
    invariant_factors_p(): boolean;
    zero_p(): boolean;
    one_p(): boolean;
}
