"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ut = __importStar(require("./util"));
const int = __importStar(require("./int"));
const cx = __importStar(require("./cell-complex"));
function boundary_vector(com, cell) {
    if (cell.dim === 0) {
        return int.vector_t.zeros(0);
    }
    else if (cell.dim === 1) {
        let edge = cell;
        let size = com.vertex_array.length;
        let vector = int.vector_t.zeros(size);
        vector.update_at(edge.start.ser(com), n => n - 1n);
        vector.update_at(edge.end.ser(com), n => n + 1n);
        return vector;
    }
    else if (cell.dim === 2) {
        let face = cell;
        let size = com.edge_array.length;
        let vector = int.vector_t.zeros(size);
        for (let e of face.circuit) {
            if (e instanceof cx.edge_rev_t) {
                vector.update_at(e.rev.ser(com), n => n - 1n);
            }
            else {
                vector.update_at(e.ser(com), n => n + 1n);
            }
        }
        return vector;
    }
    else {
        throw new Error("can only calculate dim 0, 1, 2 yet");
    }
}
exports.boundary_vector = boundary_vector;
function boundary_matrix(com, dim) {
    let m = com.dim_size(dim - 1);
    let n = com.dim_size(dim);
    let matrix = int.matrix_t.zeros(m, n);
    for (let cell of com.cell_array(dim)) {
        matrix.set_col(cell.ser(com), boundary_vector(com, cell));
    }
    return matrix;
}
exports.boundary_matrix = boundary_matrix;
function homology_diag_canonical(com, dim) {
    let low = boundary_matrix(com, dim);
    let high = boundary_matrix(com, dim + 1);
    let kernel = low.kernel();
    let image = high.image();
    let matrix = kernel.solve_matrix(image);
    if (matrix === null) {
        throw new Error("[internal] solve_matrix fail");
    }
    else {
        return matrix.diag_canonical_form();
    }
}
exports.homology_diag_canonical = homology_diag_canonical;
function euler_characteristic(com) {
    let n = 0;
    for (let d of ut.range(0, com.dim + 1)) {
        if (d % 2 === 0) {
            n += betti_number(homology_diag_canonical(com, d));
        }
        else {
            n -= betti_number(homology_diag_canonical(com, d));
        }
    }
    return n;
}
exports.euler_characteristic = euler_characteristic;
function betti_number(diag_canonical) {
    let [m, n] = diag_canonical.shape;
    let r = diag_canonical.rank();
    return m - r;
}
exports.betti_number = betti_number;
function torsion_coefficients(diag_canonical) {
    let array = new Array();
    let invariant_factors = diag_canonical.diag();
    for (let v of invariant_factors.values()) {
        let n = Number(int.abs(v));
        if (n !== 0 && n !== 1) {
            array.push(n);
        }
    }
    return array;
}
exports.torsion_coefficients = torsion_coefficients;
function report(com) {
    let obj = {};
    for (let d of ut.range(0, com.dim + 1)) {
        let diag_canonical = homology_diag_canonical(com, d);
        obj[d] = {
            betti_number: betti_number(diag_canonical),
            torsion_coefficients: torsion_coefficients(diag_canonical),
        };
    }
    obj["euler_characteristic"] = euler_characteristic(com);
    return obj;
}
exports.report = report;
//# sourceMappingURL=homology.js.map