"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const eu = __importStar(require("./euclid"));
const set_1 = require("./abstract/set");
exports.ints = new set_1.set_t({
    eq: (x, y) => x === y
});
function abs(x) {
    return x < 0n ? -x : x;
}
exports.abs = abs;
function mod(x, y) {
    let r = x % y;
    return r < 0 ? r + abs(y) : r;
}
exports.mod = mod;
function divmod(x, y) {
    let r = mod(x, y);
    let q = (x - r) / y;
    return [q, r];
}
exports.divmod = divmod;
function div(x, y) {
    let r = mod(x, y);
    return (x - r) / y;
}
exports.div = div;
function abs_lt(x, y) {
    return abs(x) < abs(y);
}
exports.abs_lt = abs_lt;
exports.ring = eu.ring({
    elements: exports.ints,
    zero: 0n,
    add: (x, y) => x + y,
    neg: (x) => -x,
    one: 1n,
    mul: (x, y) => x * y,
    degree_lt: abs_lt,
    divmod: divmod,
});
class matrix_t extends eu.matrix_t {
    constructor(the) {
        super({
            ...the,
            ring: exports.ring,
        });
    }
    copy() {
        return new matrix_t(super.copy());
    }
    static constant(n, x, y) {
        return new matrix_t(eu.matrix_t.ring_constant(exports.ring, n, x, y));
    }
    static zeros(x, y) {
        return new matrix_t(eu.matrix_t.ring_zeros(exports.ring, x, y));
    }
    static ones(x, y) {
        return new matrix_t(eu.matrix_t.ring_ones(exports.ring, x, y));
    }
    static id(n) {
        return new matrix_t(eu.matrix_t.ring_id(exports.ring, n));
    }
    transpose() {
        return new matrix_t(super.transpose());
    }
}
exports.matrix_t = matrix_t;
function matrix(array) {
    let new_array = new Array();
    for (let row of array) {
        let new_row = new Array();
        for (let x of row) {
            new_row.push(BigInt(x));
        }
        new_array.push(new_row);
    }
    return new matrix_t(eu.matrix_t.from_ring_Array2d(exports.ring, new_array));
}
exports.matrix = matrix;
class vector_t extends eu.vector_t {
    constructor(the) {
        super({
            ...the,
            ring: exports.ring,
        });
    }
    copy() {
        return new vector_t(super.copy());
    }
    static constant(n, size) {
        return new vector_t(eu.vector_t.ring_constant(exports.ring, n, size));
    }
    static zeros(size) {
        return new vector_t(eu.vector_t.ring_zeros(exports.ring, size));
    }
    static ones(size) {
        return new vector_t(eu.vector_t.ring_ones(exports.ring, size));
    }
}
exports.vector_t = vector_t;
function vector(array) {
    return new vector_t(eu.vector_t.from_ring_Array(exports.ring, array.map(BigInt)));
}
exports.vector = vector;
//# sourceMappingURL=int.js.map