import { permutation_t } from "./permutation";
export declare type Array1d = Array<number>;
export declare type Array2d = Array<Array<number>>;
export declare type Array3d = Array<Array<Array<number>>>;
export declare type index_t = Array<number>;
export declare type proj_index_t = Array<number | null>;
export declare type slice_index_t = Array<[number, number] | null>;
/**
 * strides based row-major ndarray of number
 */
export declare class array_t {
    protected buffer: Float64Array;
    readonly shape: Array<number>;
    readonly strides: Array<number>;
    readonly offset: number;
    readonly size: number;
    readonly order: number;
    /**
     * I keep the basic constructor low level,
     * and mainly use specific constructors in client modules.
     */
    constructor(buffer: Float64Array, shape: Array<number>, strides: Array<number>, offset?: number);
    static shape_to_size(shape: Array<number>): number;
    static shape_to_strides(shape: Array<number>): Array<number>;
    get_linear_index(index: index_t): number;
    linear_get(i: number): number;
    linear_set(i: number, v: number): array_t;
    get(index: index_t): number;
    set(index: index_t, x: number): array_t;
    update_at(index: index_t, f: (v: number) => number): array_t;
    values(): Generator<number, void, unknown>;
    entries(): Generator<[index_t, number], void, unknown>;
    indexes(): Generator<index_t, void, unknown>;
    reduce_with(init: number, f: (acc: number, cur: number) => number): number;
    copy(): array_t;
    proj(index: proj_index_t): array_t;
    slice(index: slice_index_t): array_t;
    /**
     * the order of `src` array can be higher or lower,
     * as long as its `.values` match `tar.indexes`.
     */
    put(index: slice_index_t, src: array_t): array_t;
    put_porj(index: proj_index_t, src: array_t): array_t;
    add(that: array_t): array_t;
    static from_1darray(array: Array1d): array_t;
    to_1darray(): Array1d;
    static from_2darray(array: Array2d): array_t;
    to_2darray(): Array2d;
    static from_3darray(array: Array3d): array_t;
    static from_buffer(buffer: Float64Array, shape: Array<number>): array_t;
    static constant(n: number, shape: Array<number>): array_t;
    static zeros(shape: Array<number>): array_t;
    static ones(shape: Array<number>): array_t;
    fill(x: number): array_t;
    static proj_index_max_p(index: proj_index_t, shape: Array<number>): boolean;
    static proj_index_inc_with_shape(index: proj_index_t, shape: Array<number>): proj_index_t;
    print(): void;
    zip(that: array_t): Generator<number[], void, unknown>;
    eq(that: array_t): boolean;
    map(f: (x: number) => number): array_t;
    for_each(f: (x: number) => any): array_t;
    append(k: number, that: array_t): array_t;
    reshape(permutation: permutation_t): array_t;
    permute(k: number, permutation: permutation_t): array_t;
    contract(that: array_t, i: number, j: number): array_t;
    static from_lower_order(lower: Array<array_t>): array_t;
}
export declare function array1d_p(array: Array<any>): boolean;
export declare function array2d_p(array: Array<any>): boolean;
export declare function array3d_p(array: Array<any>): boolean;
export declare function array(array: Array1d | Array2d | Array3d): array_t;
export declare function index_max_p(index: index_t, shape: Array<number>): boolean;
/**
 * recursive side-effect over index
 */
export declare function index_step(index: index_t, shape: Array<number>, cursor: number): index_t;
export declare function indexes_of_shape(shape: Array<number>): Generator<any[], void, unknown>;
