import * as eu from "./euclid";
import { set_t } from "./abstract/set";
export interface config_t {
    /** for almost degenerated matrix */
    epsilon: number;
}
export declare let config: config_t;
export declare function epsilon_p(x: number): boolean;
export declare function non_epsilon_p(x: number): boolean;
export declare let nums: set_t<number>;
export declare let ring: import("./abstract/ring").euclidean_ring_t<number>;
export declare function argmax(lo: number, hi: number, f: (i: number) => number): number;
export declare class matrix_t extends eu.matrix_t<number> {
    constructor(the: {
        buffer: Array<number>;
        shape: [number, number];
        strides: [number, number];
        offset?: number;
    });
    copy(): matrix_t;
    static constant(n: number, x: number, y: number): matrix_t;
    static zeros(x: number, y: number): matrix_t;
    static ones(x: number, y: number): matrix_t;
    static id(n: number): matrix_t;
    transpose(): matrix_t;
    append_cols(that: matrix_t): matrix_t;
    row_echelon_form(): matrix_t;
    /**
     * with all pivots equal to 1.
     */
    unit_row_echelon_form(): matrix_t;
    /**
     * unit row echelon form + back substitution
  
     * The reduced row echelon form of a matrix is unique
     * i.e. does not depend on the algorithm used to compute it.
     */
    reduced_row_echelon_form(): matrix_t;
    /**
     * P * A = L * U,
     * `permu.mul (this) .eq (lower.mul (upper))`,
     * (singular matrixes allowed)
     */
    lower_upper_decomposition(): {
        lower: matrix_t;
        upper: matrix_t;
        permu: matrix_t;
        inver: number;
    };
}
export declare function matrix(array: eu.Array2d<bigint | number | string>): matrix_t;
export declare class vector_t extends eu.vector_t<number> {
    constructor(the: {
        buffer: Array<number>;
        shape: [number];
        strides: [number];
        offset?: number;
    });
    copy(): vector_t;
    static constant(n: number, size: number): vector_t;
    static zeros(size: number): vector_t;
    static ones(size: number): vector_t;
}
export declare function vector(array: eu.Array1d<bigint | number | string>): vector_t;
