import * as nd from "./ndarray";
/**
 * The order matters.
 */
export declare class axes_t {
    readonly map: Map<string, axis_t>;
    readonly length: number;
    readonly shape: Array<number>;
    readonly order: number;
    constructor(map: Map<string, axis_t>);
    static from_array(array: Array<[string, axis_t]>): axes_t;
    copy(): axes_t;
    arg(name: string): number;
    name_array(): Array<string>;
    axis_array(): Array<axis_t>;
    get_name_by_index(i: number): string;
    get_axis_by_index(i: number): axis_t;
    get(name: string): axis_t;
    array_index(index: index_t): nd.index_t;
    array_index_to_index(array_index: nd.index_t): index_t;
    array_proj_index(index: index_t): nd.proj_index_t;
    array_proj_index_to_index(array_proj_index: nd.proj_index_t): index_t;
    print(): void;
}
export declare let axes: typeof axes_t.from_array;
export declare class axis_t {
    readonly map: Map<string, number>;
    readonly length: number;
    constructor(map?: Map<string, number>);
    static from_array(array: Array<string>): axis_t;
    copy(): axis_t;
    arg(name: string): number;
    arg_label(index: number): string;
    get(label: string): number;
    eq(that: axis_t): boolean;
    print(): void;
}
export declare function axis(array: Array<string>): axis_t;
export declare class index_t {
    map: Map<string, string>;
    constructor(map: Map<string, string>);
    get(name: string): string;
    static from_array(array: Array<[string, string]>): index_t;
    to_array(): Array<[string, string]>;
}
export declare let index: typeof index_t.from_array;
export declare class slice_index_t {
    map: Map<string, Array<string>>;
    constructor(map: Map<string, Array<string>>);
    get(name: string): Array<string>;
}
/**
 * ndarray + named axes,
 * where an axis maps labels to indexes.

 * Do not implicitly generate data when there are missing data.
 * Missing data should be handled explicitly.
 */
export declare class data_t {
    axes: axes_t;
    array: nd.array_t;
    readonly shape: Array<number>;
    readonly order: number;
    constructor(axes: axes_t, array: nd.array_t);
    get(index: index_t): number;
    set(index: index_t, x: number): data_t;
    update_at(index: index_t, f: (v: number) => number): data_t;
    copy(): data_t;
    entries(): Generator<[index_t, number], void, unknown>;
    add(that: data_t): data_t;
    print(): void;
    proj(index: index_t): data_t;
    contract(that: data_t, left_name: string, right_name: string): data_t;
}
export interface series_exp_t {
    [k: string]: number;
}
export declare class series_t {
    data: data_t;
    readonly name: string;
    readonly axis: axis_t;
    readonly array: nd.array_t;
    constructor(data: data_t);
    copy(): series_t;
    rename(name: string): series_t;
    label_to_index(label: string): index_t;
    get(label: string): number;
    add(that: series_t): series_t;
    entries(): Generator<[string, number], void, unknown>;
    to_exp(): series_exp_t;
    print(): void;
}
export declare function new_series(name: string, axis: axis_t, array: nd.array_t): series_t;
export declare let series: typeof new_series;
export interface frame_exp_t {
    [k: string]: series_exp_t;
}
export declare class frame_t {
    data: data_t;
    readonly array: nd.array_t;
    readonly row_name: string;
    readonly row_axis: axis_t;
    readonly col_name: string;
    readonly col_axis: axis_t;
    constructor(data: data_t, row_col_index?: [number, number]);
    copy(): frame_t;
    static from_rows(row_name: string, col_name: string, rows: Array<series_t>): frame_t;
    static from_cols(row_name: string, col_name: string, cols: Array<series_t>): frame_t;
    transpose(): frame_t;
    row(label: string): series_t;
    col(label: string): series_t;
    rows(): Generator<series_t, void, unknown>;
    row_entries(): Generator<[string, series_t], void, unknown>;
    cols(): Generator<series_t, void, unknown>;
    col_entries(): Generator<[string, series_t], void, unknown>;
    add(that: frame_t): frame_t;
    mul(that: frame_t): frame_t;
    /**
     * row is always the major.
     */
    to_exp(): frame_exp_t;
    print(): void;
}
export declare function new_frame(row_name: string, row_axis: axis_t, col_name: string, col_axis: axis_t, array: nd.array_t): frame_t;
export declare let frame: typeof new_frame;
