"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const assert_1 = __importDefault(require("assert"));
const ut = __importStar(require("./util"));
const nd = __importStar(require("./ndarray"));
/**
 * The order matters.
 */
class axes_t {
    constructor(map) {
        this.map = map;
        this.length = map.size;
        let shape = new Array();
        for (let axis of map.values()) {
            shape.push(axis.length);
        }
        this.shape = shape;
        this.order = shape.length;
    }
    static from_array(array) {
        return new axes_t(new Map(array));
    }
    copy() {
        return new axes_t(new Map(this.map));
    }
    arg(name) {
        let i = 0;
        for (let k of this.map.keys()) {
            if (k === name) {
                return i;
            }
            i += 1;
        }
        throw new Error("name not in key of map");
    }
    name_array() {
        return Array.from(this.map.keys());
    }
    axis_array() {
        return Array.from(this.map.values());
    }
    get_name_by_index(i) {
        return this.name_array()[i];
    }
    get_axis_by_index(i) {
        return this.axis_array()[i];
    }
    get(name) {
        let axis = this.map.get(name);
        if (axis === undefined) {
            throw new Error(`name: ${name} undefined`);
        }
        else {
            return axis;
        }
    }
    array_index(index) {
        let array_index = new Array();
        // the order is used here
        for (let [name, axis] of this.map) {
            array_index.push(axis.get(index.get(name)));
        }
        return array_index;
    }
    array_index_to_index(array_index) {
        let map = new Map();
        for (let k of ut.range(0, array_index.length)) {
            let i = array_index[k];
            let name = this.get_name_by_index(k);
            let axis = this.get_axis_by_index(k);
            map.set(name, axis.arg_label(i));
        }
        return new index_t(map);
    }
    array_proj_index(index) {
        let array_index = new Array();
        // the order is used here
        for (let [name, axis] of this.map) {
            if (index.map.has(name)) {
                array_index.push(axis.get(index.get(name)));
            }
            else {
                array_index.push(null);
            }
        }
        return array_index;
    }
    array_proj_index_to_index(array_proj_index) {
        let map = new Map();
        for (let k of ut.range(0, array_proj_index.length)) {
            let i = array_proj_index[k];
            if (i !== null) {
                let name = this.get_name_by_index(k);
                let axis = this.get_axis_by_index(k);
                map.set(name, axis.arg_label(i));
            }
        }
        return new index_t(map);
    }
    print() {
        for (let [name, axis] of this.map) {
            console.log(name);
            console.table(axis.map);
        }
    }
}
exports.axes_t = axes_t;
exports.axes = axes_t.from_array;
class axis_t {
    constructor(map = new Map()) {
        this.map = map;
        this.length = map.size;
    }
    static from_array(array) {
        let map = new Map();
        for (let i of ut.range(0, array.length)) {
            let v = array[i];
            map.set(v, i);
        }
        return new axis_t(map);
    }
    copy() {
        return new axis_t(new Map(this.map));
    }
    arg(name) {
        let i = 0;
        for (let k of this.map.keys()) {
            if (k === name) {
                return i;
            }
            i += 1;
        }
        throw new Error("name not in key of map");
    }
    arg_label(index) {
        for (let [label, i] of this.map) {
            if (i === index) {
                return label;
            }
        }
        throw new Error("label not in key of map");
    }
    get(label) {
        let i = this.map.get(label);
        if (i === undefined) {
            throw new Error("label undefined");
        }
        else {
            return i;
        }
    }
    eq(that) {
        if (this.length !== that.length) {
            return false;
        }
        else {
            return _.isEqual(this.map, that.map);
        }
    }
    print() {
        console.table(this.map);
    }
}
exports.axis_t = axis_t;
function axis(array) {
    return axis_t.from_array(array);
}
exports.axis = axis;
class index_t {
    constructor(map) {
        this.map = map;
    }
    get(name) {
        let label = this.map.get(name);
        if (label === undefined) {
            throw new Error(`name: ${name} undefined`);
        }
        else {
            return label;
        }
    }
    static from_array(array) {
        return new index_t(new Map(array));
    }
    to_array() {
        return Array.from(this.map);
    }
}
exports.index_t = index_t;
exports.index = index_t.from_array;
class slice_index_t {
    constructor(map) {
        this.map = map;
    }
    get(name) {
        let label_array = this.map.get(name);
        if (label_array === undefined) {
            throw new Error(`name: ${name} undefined`);
        }
        else {
            return label_array;
        }
    }
}
exports.slice_index_t = slice_index_t;
/**
 * ndarray + named axes,
 * where an axis maps labels to indexes.

 * Do not implicitly generate data when there are missing data.
 * Missing data should be handled explicitly.
 */
class data_t {
    constructor(axes, array) {
        if (!_.isEqual(axes.shape, array.shape)) {
            console.log("axes.shape:", axes.shape);
            console.log("array.shape:", array.shape);
            throw new Error("shape mismatch");
        }
        this.axes = axes;
        this.array = array;
        this.shape = axes.shape;
        this.order = axes.order;
    }
    get(index) {
        let array_index = this.axes.array_index(index);
        return this.array.get(array_index);
    }
    set(index, x) {
        let array_index = this.axes.array_index(index);
        this.array.set(array_index, x);
        return this;
    }
    update_at(index, f) {
        return this.set(index, f(this.get(index)));
    }
    copy() {
        return new data_t(this.axes.copy(), this.array.copy());
    }
    *entries() {
        for (let [array_index, v] of this.array.entries()) {
            let index = this.axes.array_index_to_index(array_index);
            yield [index, v];
        }
    }
    add(that) {
        let data = this.copy();
        for (let [i, v] of that.entries()) {
            data.update_at(i, x => x + v);
        }
        return data;
    }
    print() {
        if (this.order === 1) {
            new series_t(this).print();
        }
        else if (this.order === 2) {
            new frame_t(this).print();
        }
        else {
            let array_proj_index = this.shape.slice().fill(0);
            array_proj_index.pop();
            array_proj_index.pop();
            array_proj_index.push(null);
            array_proj_index.push(null);
            while (true) {
                let index = this.axes.array_proj_index_to_index(array_proj_index);
                console.log("data index:", index.to_array());
                console.log("array index:", array_proj_index);
                this.proj(index).print();
                array_proj_index =
                    nd.array_t.proj_index_inc_with_shape(array_proj_index, this.shape);
                if (nd.array_t.proj_index_max_p(array_proj_index, this.shape)) {
                    console.log("data index:", index.to_array());
                    console.log("array index:", array_proj_index);
                    this.proj(index).print();
                    return;
                }
            }
        }
    }
    proj(index) {
        let axes_array = new Array();
        for (let [name, axis] of this.axes.map) {
            if (!index.map.has(name)) {
                axes_array.push([name, axis]);
            }
        }
        return new data_t(axes_t.from_array(axes_array), this.array.proj(this.axes.array_proj_index(index)));
    }
    // TODO
    // slice (index: slice_index_t): data_t {
    // }
    contract(that, left_name, right_name) {
        let left_arg = this.axes.arg(left_name);
        let right_arg = that.axes.arg(right_name);
        let array = this.array.contract(that.array, left_arg, right_arg);
        let map = new Map();
        for (let [name, axis] of this.axes.map) {
            if (name !== left_name) {
                map.set(name, axis);
            }
        }
        for (let [name, axis] of that.axes.map) {
            if (name !== right_name) {
                map.set(name, axis);
            }
        }
        let axes = new axes_t(map);
        return new data_t(axes, array);
    }
}
exports.data_t = data_t;
class series_t {
    constructor(data) {
        assert_1.default(data.order === 1);
        this.data = data;
        this.name = data.axes.get_name_by_index(0);
        this.axis = data.axes.get_axis_by_index(0);
        this.array = data.array;
    }
    copy() {
        return new series_t(this.data);
    }
    rename(name) {
        return new_series(name, this.axis, this.array);
    }
    label_to_index(label) {
        return index_t.from_array([
            [this.name, label],
        ]);
    }
    get(label) {
        return this.data.get(this.label_to_index(label));
    }
    add(that) {
        return new series_t(this.data.add(that.data));
    }
    *entries() {
        for (let label of this.axis.map.keys()) {
            yield [label, this.get(label)];
        }
    }
    to_exp() {
        let exp = {};
        for (let [k, v] of this.entries()) {
            exp[k] = v;
        }
        return exp;
    }
    print() {
        console.group(`axis_name: ${this.name}`);
        console.table(this.to_exp());
        console.groupEnd();
    }
}
exports.series_t = series_t;
function new_series(name, axis, array) {
    let axes = axes_t.from_array([
        [name, axis],
    ]);
    return new series_t(new data_t(axes, array));
}
exports.new_series = new_series;
exports.series = new_series;
class frame_t {
    constructor(data, row_col_index = [0, 1]) {
        assert_1.default(data.order === 2);
        this.data = data;
        this.array = data.array;
        let [row_index, col_index] = row_col_index;
        this.row_name = data.axes.get_name_by_index(row_index);
        this.row_axis = data.axes.get_axis_by_index(row_index);
        this.col_name = data.axes.get_name_by_index(col_index);
        this.col_axis = data.axes.get_axis_by_index(col_index);
    }
    copy() {
        return new frame_t(this.data);
    }
    static from_rows(row_name, col_name, rows) {
        assert_1.default(rows.length !== 0);
        let first_row = rows[0];
        let label_array = new Array();
        let col_axis = first_row.axis;
        let lower = new Array();
        for (let row of rows) {
            assert_1.default(row.axis.eq(col_axis));
            label_array.push(row.name);
            lower.push(row.array);
        }
        let row_axis = axis_t.from_array(label_array);
        return new_frame(row_name, row_axis, col_name, col_axis, nd.array_t.from_lower_order(lower));
    }
    static from_cols(row_name, col_name, cols) {
        return frame_t.from_rows(col_name, row_name, cols).transpose();
    }
    transpose() {
        return new frame_t(this.data, [1, 0]);
    }
    row(label) {
        return new series_t(this.data.proj(index_t.from_array([
            [this.row_name, label],
        ])));
    }
    col(label) {
        return new series_t(this.data.proj(index_t.from_array([
            [this.col_name, label],
        ])));
    }
    *rows() {
        for (let label of this.row_axis.map.keys()) {
            yield this.row(label);
        }
    }
    *row_entries() {
        for (let label of this.row_axis.map.keys()) {
            yield [label, this.row(label)];
        }
    }
    *cols() {
        for (let label of this.col_axis.map.keys()) {
            yield this.col(label);
        }
    }
    *col_entries() {
        for (let label of this.col_axis.map.keys()) {
            yield [label, this.col(label)];
        }
    }
    add(that) {
        return new frame_t(this.data.add(that.data));
    }
    mul(that) {
        return new frame_t(this.data.contract(that.data, this.col_name, that.row_name));
    }
    /**
     * row is always the major.
     */
    to_exp() {
        let exp = {};
        for (let [label, row] of this.row_entries()) {
            exp[label] = row.to_exp();
        }
        return exp;
    }
    print() {
        console.log(`row_name: ${this.row_name}`);
        console.log(`col_name: ${this.col_name}`);
        console.group();
        console.table(this.to_exp());
        console.groupEnd();
    }
}
exports.frame_t = frame_t;
function new_frame(row_name, row_axis, col_name, col_axis, array) {
    let axes = axes_t.from_array([
        [row_name, row_axis],
        [col_name, col_axis],
    ]);
    return new frame_t(new data_t(axes, array));
}
exports.new_frame = new_frame;
exports.frame = new_frame;
//# sourceMappingURL=panel-data.js.map