"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = __importStar(require("lodash"));
const assert_1 = __importDefault(require("assert"));
const ut = __importStar(require("./util"));
class permutation_t {
    constructor(sequence) {
        this.sequence = sequence;
        this.size = sequence.length;
    }
    get(i) {
        return this.sequence[i];
    }
    *[Symbol.iterator]() {
        for (let v of this.sequence) {
            yield v;
        }
    }
    *pairs() {
        let i = 0;
        for (let x of this.sequence) {
            yield [i, x];
            i += 1;
        }
    }
    *inv_pairs() {
        let i = 0;
        for (let x of this.sequence) {
            yield [x, i];
            i += 1;
        }
    }
    inv() {
        let sequence = Array.from(this.inv_pairs())
            .sort((x, y) => x[0] - y[0])
            .map(x => x[1]);
        return new permutation_t(sequence);
    }
    mul(that) {
        let sequence = new Array();
        for (let i of that.sequence) {
            sequence.push(this.sequence[i]);
        }
        return new permutation_t(sequence);
    }
    eq(that) {
        return _.isEqual(this.sequence, that.sequence);
    }
    lt(that) {
        assert_1.default(this.size === that.size);
        return this.lt_after(that, 0);
    }
    lt_after(that, n) {
        if (n >= this.size) {
            return false;
        }
        let x = this.get(n);
        let y = that.get(n);
        if (x < y) {
            return true;
        }
        else if (x === y) {
            return this.lt_after(that, n + 1);
        }
        else {
            return false;
        }
    }
    lte(that) {
        return this.lte_after(that, 0);
    }
    lte_after(that, n) {
        if (n >= this.size) {
            return true;
        }
        let x = this.get(n);
        let y = that.get(n);
        if (x < y) {
            return true;
        }
        else if (x === y) {
            return this.lte_after(that, n + 1);
        }
        else {
            return false;
        }
    }
    gt(that) {
        return that.lt(this);
    }
    gte(that) {
        return that.lte(this);
    }
    static id(size) {
        return new permutation_t(Array.from(ut.range(0, size)));
    }
    tuck(i, j) {
        let sequence = new Array();
        for (let k of ut.range(0, this.size)) {
            if (i <= k && k < j) {
                sequence[k] = this.get(k + 1);
            }
            else if (k === j) {
                sequence[k] = this.get(i);
            }
            else {
                sequence[k] = this.get(k);
            }
        }
        return new permutation_t(sequence);
    }
}
exports.permutation_t = permutation_t;
//# sourceMappingURL=permutation.js.map